@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -36,6 +36,7 @@ from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import (
     getSupportedPythonVersions,
+    isStaticallyLinkedPython,
     isUninstalledPython,
     python_version,
     python_version_str
@@ -113,9 +114,42 @@ def createNodeTree(filename):
             pattern = pattern
         )
 
+    for package_name in Options.getMustIncludePackages():
+        package_package, package_directory, kind = Importing.findModule(
+            importing      = None,
+            module_name    = package_name,
+            parent_package = None,
+            level          = 0,
+            warn           = False
+        )
+
+        if kind != "absolute":
+            sys.exit("Error, failed to locate package %r." % package_name)
+
+        Recursion.checkPluginPath(
+            plugin_filename = package_directory,
+            module_package  = package_package
+        )
+
+    for module_name in Options.getMustIncludeModules():
+        module_package, module_filename, kind = Importing.findModule(
+            importing      = None,
+            module_name    = module_name,
+            parent_package = None,
+            level          = 0,
+            warn           = False
+        )
+
+        if kind != "absolute":
+            sys.exit("Error, failed to locate module %r." % module_name)
+
+        Recursion.checkPluginSinglePath(
+            plugin_filename = module_filename,
+            module_package  = module_package
+        )
 
     # Then optimize the tree and potentially recursed modules.
-    Optimization.optimize()
+    Optimization.optimize(main_module.getOutputFilename())
 
     if Options.isExperimental("check_xml_persistence"):
         for module in ModuleRegistry.getRootModules():
@@ -436,9 +470,13 @@ def makeSourceDirectory(main_module):
     )
 
 
+def _asBoolStr(value):
+    return "true" if value else "false"
+
+
 def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
-    # have checks for them, leading to many branches, pylint: disable=too-many-branches
+    # have checks for them, leading to many branches, pylint: disable=too-many-branches,too-many-statements
 
     if hasattr(sys, "abiflags"):
         abiflags = sys.abiflags
@@ -450,22 +488,19 @@ def runScons(main_module, quiet):
     else:
         abiflags = None
 
-    def asBoolStr(value):
-        return "true" if value else "false"
-
     options = {
         "name"            : os.path.basename(
             getTreeFilenameWithSuffix(main_module, "")
         ),
         "result_name"     : getResultBasepath(main_module),
         "source_dir"      : getSourceDirectoryPath(main_module),
-        "debug_mode"      : asBoolStr(Options.isDebug()),
-        "python_debug"    : asBoolStr(Options.isPythonDebug()),
-        "unstripped_mode" : asBoolStr(Options.isUnstripped()),
-        "module_mode"     : asBoolStr(Options.shallMakeModule()),
-        "full_compat"     : asBoolStr(Options.isFullCompat()),
+        "debug_mode"      : _asBoolStr(Options.isDebug()),
+        "python_debug"    : _asBoolStr(Options.isPythonDebug()),
+        "unstripped_mode" : _asBoolStr(Options.isUnstripped()),
+        "module_mode"     : _asBoolStr(Options.shallMakeModule()),
+        "full_compat"     : _asBoolStr(Options.isFullCompat()),
         "experimental"    : ','.join(Options.getExperimentalIndications()),
-        "trace_mode"      : asBoolStr(Options.shallTraceExecution()),
+        "trace_mode"      : _asBoolStr(Options.shallTraceExecution()),
         "python_version"  : python_version_str,
         "target_arch"     : Utils.getArchitecture(),
         "python_prefix"   : sys.prefix,
@@ -497,6 +532,9 @@ def runScons(main_module, quiet):
        isUninstalledPython():
         options["uninstalled_python"] = "true"
 
+    if isStaticallyLinkedPython():
+        options["static_python"] = "true"
+
     if ModuleRegistry.getUncompiledTechnicalModules():
         options["frozen_modules"] = str(
             len(ModuleRegistry.getUncompiledTechnicalModules())
@@ -547,6 +585,9 @@ def runScons(main_module, quiet):
     if python_version < 300 and sys.flags.unicode:
         options["python_sysflag_unicode"] = "true"
 
+    if python_version >= 370 and sys.flags.utf8_mode:
+        options["python_sysflag_utf8"] = "true"
+
     if abiflags:
         options["abiflags"] = abiflags
 
@@ -706,7 +747,8 @@ def handleSyntaxError(e):
 
 Nuitka is very syntax compatible with standard Python. It is currently running
 with Python version '%s', you might want to specify more clearly with the use
-of e.g. '--python-version=%s' option, if that's not the one expected.
+of the precise Python interpreter binary and '-m nuitka', e.g. use this
+'python%s -m nuitka' option, if that's not the one the program expects.
 """ % (python_version_str, suggested_python_version_str)
 
     sys.exit(error_message)
@@ -783,13 +825,6 @@ def main():
 
             sys.exit(0)
 
-        # Remove the source directory (now build directory too) if asked to.
-        if Options.isRemoveBuildDir():
-            removeDirectory(
-                path          = getSourceDirectoryPath(main_module),
-                ignore_errors = False
-            )
-
         if Options.isStandaloneMode():
             binary_filename = options["result_name"] + ".exe"
 
@@ -811,6 +846,7 @@ def main():
                 )
 
             copyUsedDLLs(
+                source_dir              = getSourceDirectoryPath(main_module),
                 dist_dir                = dist_dir,
                 standalone_entry_points = standalone_entry_points
             )
@@ -833,6 +869,13 @@ def main():
                     target_filename
                 )
 
+        # Remove the source directory (now build directory too) if asked to.
+        if Options.isRemoveBuildDir():
+            removeDirectory(
+                path          = getSourceDirectoryPath(main_module),
+                ignore_errors = False
+            )
+
         # Modules should not be executable, but Scons creates them like it, fix
         # it up here.
         if Utils.getOS() != "Windows" and Options.shallMakeModule():
