@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,14 +22,8 @@ Attribute lookup, setting.
 
 from nuitka import Options
 
-from .ConstantCodes import getConstantCode
-from .ErrorCodes import (
-    getErrorExitBoolCode,
-    getErrorExitCode,
-    getReleaseCode,
-    getReleaseCodes
-)
-from .Helpers import generateChildExpressionsCode, generateExpressionCode
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
+from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode
 from .LabelCodes import getBranchingCode
 from .PythonAPICodes import generateCAPIObjectCode, generateCAPIObjectCode0
 
@@ -40,7 +34,6 @@ def generateAssignmentAttributeCode(statement, emit, context):
     value          = statement.getAssignSource()
 
     value_name = context.allocateTempName("assattr_name")
-
     generateExpressionCode(
         to_name    = value_name,
         expression = value,
@@ -80,8 +73,7 @@ def generateAssignmentAttributeCode(statement, emit, context):
         getAttributeAssignmentCode(
             target_name    = target_name,
             value_name     = value_name,
-            attribute_name = getConstantCode(
-                context  = context,
+            attribute_name = context.getConstantCode(
                 constant = attribute_name
             ),
             emit           = emit,
@@ -109,8 +101,7 @@ def generateDelAttributeCode(statement, emit, context):
 
     getAttributeDelCode(
         target_name    = target_name,
-        attribute_name = getConstantCode(
-            context  = context,
+        attribute_name = context.getConstantCode(
             constant = statement.getAttributeName()
         ),
         emit           = emit,
@@ -163,26 +154,18 @@ def getAttributeLookupCode(to_name, source_name, attribute_name, needs_check,
             "%s = LOOKUP_ATTRIBUTE( %s, %s );" % (
                 to_name,
                 source_name,
-                getConstantCode(
-                    context  = context,
-                    constant = attribute_name
-                )
+                context.getConstantCode(attribute_name)
             )
         )
 
-    getReleaseCode(
+    getErrorExitCode(
+        check_name   = to_name,
         release_name = source_name,
+        needs_check  = needs_check,
         emit         = emit,
         context      = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -197,19 +180,14 @@ def getAttributeCheckBoolCode(source_name, attr_name, needs_check, emit, context
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == -1" % res_name,
         release_names = (source_name, attr_name),
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition   = "%s == -1" % res_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     getBranchingCode("%s == 1" % res_name, emit, context)
 
 
@@ -227,19 +205,13 @@ def getAttributeAssignmentCode(target_name, attribute_name, value_name, emit,
     )
 
     getErrorExitBoolCode(
-        condition = "%s == false" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
-    getReleaseCodes(
+        condition     = "%s == false" % res_name,
         release_names = (value_name, target_name, attribute_name),
         emit          = emit,
         context       = context
     )
 
 
-
 def getAttributeAssignmentDictSlotCode(target_name, value_name, emit, context):
     """ Code for special case target.__dict__ = value """
 
@@ -254,12 +226,7 @@ def getAttributeAssignmentDictSlotCode(target_name, value_name, emit, context):
     )
 
     getErrorExitBoolCode(
-        condition = "%s == false" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
-    getReleaseCodes(
+        condition     = "%s == false" % res_name,
         release_names = (value_name, target_name),
         emit          = emit,
         context       = context
@@ -280,12 +247,7 @@ def getAttributeAssignmentClassSlotCode(target_name, value_name, emit, context):
     )
 
     getErrorExitBoolCode(
-        condition = "%s == false" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
-    getReleaseCodes(
+        condition     = "%s == false" % res_name,
         release_names = (value_name, target_name),
         emit          = emit,
         context       = context
@@ -303,18 +265,13 @@ def getAttributeDelCode(target_name, attribute_name, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitBoolCode(
+        condition     = "%s == -1" % res_name,
         release_names = (target_name, attribute_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitBoolCode(
-        condition = "%s == -1" % res_name,
-        emit      = emit,
-        context   = context
-    )
-
 
 def generateAttributeLookupSpecialCode(to_name, expression, emit, context):
     source_name, = generateChildExpressionsCode(
@@ -328,8 +285,7 @@ def generateAttributeLookupSpecialCode(to_name, expression, emit, context):
     getAttributeLookupSpecialCode(
         to_name     = to_name,
         source_name = source_name,
-        attr_name   = getConstantCode(
-            context  = context,
+        attr_name   = context.getConstantCode(
             constant = attribute_name
         ),
         needs_check = expression.getLookupSource().mayRaiseExceptionAttributeLookupSpecial(
@@ -351,19 +307,14 @@ def getAttributeLookupSpecialCode(to_name, source_name, attr_name, needs_check,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (source_name, attr_name),
         emit          = emit,
+        needs_check   = needs_check,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        emit        = emit,
-        needs_check = needs_check,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
