@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,9 +23,9 @@ able to execute them without creating the argument dictionary at all.
 
 """
 
+from .CodeHelpers import generateChildExpressionCode, generateExpressionCode
 from .ConstantCodes import getConstantAccess
-from .ErrorCodes import getErrorExitCode, getReleaseCode, getReleaseCodes
-from .Helpers import generateChildExpressionCode, generateExpressionCode
+from .ErrorCodes import getErrorExitCode
 from .LineNumberCodes import emitLineNumberUpdateCode
 from .templates.CodeTemplatesCalls import (
     template_call_function_with_args_decl,
@@ -335,19 +335,14 @@ def getCallCodeNoArgs(to_name, called_name, needs_check, emit, context):
         )
     )
 
-    getReleaseCode(
+    getErrorExitCode(
+        check_name   = to_name,
         release_name = called_name,
         emit         = emit,
+        needs_check  = needs_check,
         context      = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        emit        = emit,
-        needs_check = needs_check,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -363,22 +358,17 @@ def getInstanceCallCodeNoArgs(to_name, called_name, called_attribute_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (
             called_name,
             called_attribute_name
         ),
         emit          = emit,
+        needs_check   = needs_check,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        emit        = emit,
-        needs_check = needs_check,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -412,19 +402,14 @@ def getInstanceCallCodePosArgsQuick(to_name, called_name, called_attribute_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = [called_name] + arg_names,
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -453,19 +438,14 @@ def getCallCodePosArgsQuick(to_name, called_name, arg_names, needs_check,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = [called_name] + arg_names,
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -490,22 +470,17 @@ def getInstanceCallCodeFromTuple(to_name, called_name, called_attribute_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (
             called_name,
             called_attribute_name
         ),
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -529,19 +504,14 @@ def getCallCodeFromTuple(to_name, called_name, arg_tuple, arg_size,
         )
     )
 
-    getReleaseCode(
+    getErrorExitCode(
+        check_name   = to_name,
         release_name = called_name,
+        needs_check  = needs_check,
         emit         = emit,
         context      = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -558,19 +528,14 @@ def getInstanceCallCodePosArgs(to_name, called_name, called_attribute_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (called_name, args_name),
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -586,19 +551,14 @@ def getCallCodePosArgs(to_name, called_name, args_name, needs_check, emit,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (called_name, args_name),
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name  = to_name,
-        needs_check = needs_check,
-        emit        = emit,
-        context     = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -613,18 +573,13 @@ def _getCallCodeKeywordArgs(to_name, called_name, call_kw_name, emit, context):
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (called_name, call_kw_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
@@ -641,18 +596,13 @@ def getCallCodePosKeywordArgs(to_name, called_name, call_args_name,
         )
     )
 
-    getReleaseCodes(
+    getErrorExitCode(
+        check_name    = to_name,
         release_names = (called_name, call_args_name, call_kw_name),
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
     context.addCleanupTempName(to_name)
 
 
