@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -41,8 +41,13 @@ from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
     unicode,
     xrange
 )
-from nuitka.Constants import compareConstants, getConstantWeight, isMutable
-from nuitka.PythonVersions import python_version
+from nuitka.Builtins import builtin_named_values, builtin_named_values_list
+from nuitka.Constants import (
+    NoneType,
+    compareConstants,
+    getConstantWeight,
+    isMutable
+)
 
 from .BlobCodes import StreamData
 from .Emission import SourceCodeCollector
@@ -298,20 +303,22 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
         This may be module or global init. Code makes sure that nested
         constants belong into the same scope.
     """
+    # Got a couple of values to dodge, pylint: disable=too-many-return-statements
 
     if constant_value is None:
         return
-    if constant_value is False:
+    elif constant_value is False:
         return
-    if constant_value is True:
+    elif constant_value is True:
         return
-    if constant_value is Ellipsis:
+    elif constant_value is Ellipsis:
         return
-    if type(constant_value) is type:
+    elif constant_value is NotImplemented:
         return
-
-    # Do not repeat ourselves.
-    if constant_identifier in done:
+    elif type(constant_value) is type:
+        return
+    elif constant_identifier in done:
+        # Do not repeat ourselves.
         return
 
     if Options.shallTraceExecution():
@@ -321,6 +328,8 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
     __addConstantInitCode(context, emit, check, constant_type, constant_value,
                           constant_identifier, module_level)
 
+    # In debug mode, lets check if the constants somehow change behind our
+    # back, add those values too.
     if Options.isDebug():
         emit(
              """\
@@ -365,7 +374,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
     if constant_type is long:
         # See above, same for long values. Note: These are of course not
         # existent with Python3 which would have covered it before.
-        if constant_value >= 0 and constant_value <= max_unsigned_long:
+        if 0 <= constant_value <= max_unsigned_long:
             emit (
                 "%s = PyLong_FromUnsignedLong( %sul );" % (
                     constant_identifier,
@@ -374,7 +383,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
             )
 
             return
-        elif constant_value < 0 and constant_value >= min_signed_long:
+        elif 0 > constant_value >= min_signed_long:
             emit (
                 "%s = PyLong_FromLong( %sl );" % (
                     constant_identifier,
@@ -660,6 +669,17 @@ CHECK_OBJECT( const_int_pos_1 );
         if attemptToMarshal(constant_identifier, constant_value, emit):
             return
 
+        # Special handling for empty frozensets.
+        if not constant_value and constant_type is frozenset:
+            emit(
+                "%s = PyObject_CallFunction((PyObject*)&PyFrozenSet_Type, NULL);" % (
+                    constant_identifier,
+                )
+            )
+
+            return
+
+
         # TODO: Hinting size is really not possible?
         emit(
             "%s = %s( NULL );" % (
@@ -834,6 +854,30 @@ CHECK_OBJECT( const_int_pos_1 );
 
         return
 
+    if constant_value in builtin_named_values_list:
+        builtin_name = builtin_named_values[constant_value]
+        builtin_identifier = context.getConstantCode(builtin_name)
+
+        _addConstantInitCode(
+            emit                = emit,
+            check               = check,
+            constant_type       = type(builtin_name),
+            constant_value      = builtin_name,
+            constant_identifier = builtin_identifier,
+            module_level        = module_level,
+            context             = context
+        )
+
+        emit(
+            "%s = LOOKUP_BUILTIN( %s );" % (
+                constant_identifier,
+                builtin_identifier
+            )
+        )
+
+        return
+
+
     # Must not reach this, if we did, it's in error, and we need to know.
     assert False, (type(constant_value), constant_value, constant_identifier)
 
@@ -882,6 +926,8 @@ def getConstantsDeclCode(context):
             continue
         if constant_value is Ellipsis:
             continue
+        if constant_value is NotImplemented:
+            continue
         if type(constant_value) is type:
             continue
 
@@ -1045,7 +1091,7 @@ def getConstantInitCodes(module_context):
 
 
 def allocateNestedConstants(module_context):
-    # Lots of types to deal with, pylint: disable=too-many-branches
+    # Lots of types to deal with.
 
     def considerForDeferral(constant_value):
         module_context.getConstantCode(constant_value)
@@ -1068,25 +1114,12 @@ def allocateNestedConstants(module_context):
             considerForDeferral(constant_value.stop)
         elif constant_type is xrange:
             if xrange is range:
-                if python_version >= 330:
-                    # For Python2 ranges, we use C long values directly.
-                    considerForDeferral(constant_value.start)
-                    considerForDeferral(constant_value.step)
-                    considerForDeferral(constant_value.stop)
-                else:
-                    parts = [
-                        int(value)
-                        for value in
-                        str(constant_value)[6:-1].split(',')
-                    ]
-
-                    if len(parts) <= 1:
-                        parts.append(0)
-                    if len(parts) <= 2:
-                        parts.append(1)
-
-                    for value in parts:
-                        considerForDeferral(value)
+                # For Python2 ranges, we use C long values directly.
+                considerForDeferral(constant_value.start)
+                considerForDeferral(constant_value.step)
+                considerForDeferral(constant_value.stop)
+        elif constant_value in builtin_named_values_list:
+            considerForDeferral(builtin_named_values[constant_value])
 
     for constant_identifier in set(module_context.getConstants()):
         constant_value = module_context.global_context.constants[
@@ -1097,6 +1130,10 @@ def allocateNestedConstants(module_context):
 
         if constant_type in (tuple, dict, list, set, frozenset, slice, xrange):
             considerForDeferral(constant_value)
+        elif constant_type in (str, NoneType, int, long):
+            pass
+        elif constant_value in builtin_named_values_list:
+            considerForDeferral(builtin_named_values[constant_value])
 
 
 def getConstantsDefinitionCode(context):
