@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,11 +24,14 @@ of frames for different uses.
 from nuitka.PythonVersions import python_version
 
 from . import Emission
+from .CodeHelpers import _generateStatementSequenceCode
+from .ErrorCodes import getFrameVariableTypeDescriptionCode
 from .ExceptionCodes import getTracebackMakingIdentifier
-from .Helpers import _generateStatementSequenceCode
 from .Indentation import indented
+from .LabelCodes import getGotoCode, getLabelCode
 from .ModuleCodes import getModuleAccessCode
 from .templates.CodeTemplatesFrames import (
+    template_frame_attach_locals,
     template_frame_guard_cache_decl,
     template_frame_guard_frame_decl,
     template_frame_guard_full_block,
@@ -59,15 +62,25 @@ def getFrameLocalsStorageSize(type_descriptions):
     result = candidates.pop()
 
     while candidates:
+        # assert False, (type_descriptions, context.frame_variables_stack[-1])
+
         result = "MAX( %s, %s )" % (result, candidates.pop())
 
     return result
 
 
+def _searchLocalVariableByName(local_variables, variable_name):
+    for local_variable in local_variables:
+        if local_variable.getName() == variable_name:
+            return local_variable
+
+    return None
+
+
 def generateStatementsFrameCode(statement_sequence, emit, context):
     # This is a wrapper that provides also handling of frames, which got a
-    # lot of variants and details, therefore lots of branches.
-    # pylint: disable=too-many-branches,too-many-locals
+    # lot of variants and details, therefore lots of branches and details.
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     context.pushCleanupScope()
 
@@ -80,15 +93,18 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
 
     parent_exception_exit = context.getExceptionEscape()
 
-    # Allow stacking of frame handles.
-    old_frame_handle = context.getFrameHandle()
+    # For nested frames, make sure to restore set the type description.
+    if context.getFrameHandle() is not None:
+        real_parent_exception_exit = parent_exception_exit
+        parent_exception_exit = context.allocateLabel("nested_frame_exit")
 
     if statement_sequence.hasStructureMember():
         frame_identifier = "%s->m_frame" % context.getContextObjectName()
     else:
         frame_identifier = code_identifier.replace("codeobj_", "frame_")
 
-    context.setFrameHandle(
+    # Allow stacking of frame handles.
+    frame_identifier = context.pushFrameHandle(
         frame_identifier
     )
 
@@ -107,12 +123,18 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     else:
         parent_return_exit = None
 
-    pushed_frame_variables = statement_sequence.mayRaiseException(BaseException)
+    # We need to define that all the variables needs to be pushed. We do not
+    # have a flag that says "always NULL" for variables. With efficient NULL
+    # passing however (not at all, TODO), that doesn't matter much.
+    local_variables = statement_sequence.getParentVariableProvider().getLocalVariables()
 
-    if pushed_frame_variables:
-        context.pushFrameVariables(
+    context.pushFrameVariables(
+        tuple(
+            _searchLocalVariableByName(local_variables, variable_name)
+            for variable_name in
             code_object.getVarNames()
         )
+    )
 
     # Now generate the statements code into a local buffer, to we can wrap
     # the frame stuff around it.
@@ -181,25 +203,31 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     else:
         assert False, guard_mode
 
-    if pushed_frame_variables:
-        context.popFrameVariables()
+    context.popFrameVariables()
+    context.popFrameHandle()
+
+    # For nested frames, make sure to restore set the type description.
+    if context.getFrameHandle() is not None:
+        label = context.allocateLabel("skip_nested_handling")
+        getGotoCode(label, emit)
+        getLabelCode(parent_exception_exit, emit)
+        emit(getFrameVariableTypeDescriptionCode(context))
+        getGotoCode(real_parent_exception_exit, emit)
+        getLabelCode(label, emit)
+
+        parent_exception_exit = real_parent_exception_exit
+
+    context.popCleanupScope()
 
     context.setExceptionEscape(parent_exception_exit)
 
     if frame_return_exit is not None:
         context.setReturnTarget(parent_return_exit)
 
-    context.setFrameHandle(old_frame_handle)
-
-    context.popCleanupScope()
 
 
 def getTypeSizeOf(type_indicator):
-    if type_indicator in ('O', 'o'):
-        return "sizeof(PyObject *)"
-    elif type_indicator == 'c':
-        return "sizeof(struct Nuitka_CellObject *)"
-    elif type_indicator == 'N':
+    if type_indicator in ('O', 'o', 'N', 'c'):
         return "sizeof(void *)"
     elif type_indicator == 'b':
         return "sizeof(nuitka_bool)"
@@ -207,9 +235,22 @@ def getTypeSizeOf(type_indicator):
         assert False, type_indicator
 
 
+def getFrameAttachLocalsCode(context, frame_identifier):
+    frame_variable_codes = context.getFrameVariableCodeNames()
+    frame_variable_codes = ",\n    ".join(frame_variable_codes)
+    if frame_variable_codes:
+        frame_variable_codes = ",\n    " + frame_variable_codes
+
+
+    return template_frame_attach_locals % {
+        "frame_identifier"      : frame_identifier,
+        "type_description"      : context.getFrameVariableTypeDescriptionName(),
+        "frame_variable_refs"   : frame_variable_codes
+    }
+
+
 def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
-                           type_descriptions,
-                           needs_preserve, parent_exception_exit,
+                           type_descriptions, needs_preserve, parent_exception_exit,
                            parent_return_exit, frame_exception_exit,
                            frame_return_exit, emit, context):
     # We really need this many parameters here.
@@ -233,7 +274,7 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
             "code_identifier"   : code_identifier,
             "locals_size"       : getFrameLocalsStorageSize(type_descriptions),
             "codes"             : indented(codes, 0),
-            "module_identifier" : getModuleAccessCode(context = context),
+            "module_identifier" : getModuleAccessCode(context),
             "no_exception_exit" : no_exception_exit,
             "needs_preserve"    : 1 if needs_preserve else 0,
         }
@@ -250,12 +291,6 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
         )
 
     if frame_exception_exit is not None:
-        frame_variable_codes = context.getFrameVariableCodeNames()
-
-        if frame_variable_codes:
-            frame_variable_codes = ',' + frame_variable_codes
-        else:
-            frame_variable_codes = ""
 
         emit(
             template_frame_guard_full_exception_handler % {
@@ -266,8 +301,7 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
                                           ),
                 "parent_exception_exit" : parent_exception_exit,
                 "frame_exception_exit"  : frame_exception_exit,
-                "type_description"      : "type_description" if context.needsFrameVariableTypeDescription() else '""',
-                "frame_variable_refs"   : frame_variable_codes,
+                "attach_locals"         : getFrameAttachLocalsCode(context, frame_identifier),
                 "needs_preserve"        : 1 if needs_preserve else 0,
             }
         )
@@ -295,7 +329,7 @@ def getFrameGuardOnceCode(frame_identifier, code_identifier,
             "frame_identifier"      : frame_identifier,
             "code_identifier"       : code_identifier,
             "codes"                 : indented(codes, 0),
-            "module_identifier"     : getModuleAccessCode(context = context),
+            "module_identifier"     : getModuleAccessCode(context),
             "tb_making"             : getTracebackMakingIdentifier(
                                          context     = context,
                                          lineno_name = "exception_lineno"
@@ -317,24 +351,25 @@ def getFrameGuardLightCode(code_identifier, codes, parent_exception_exit,
     context.markAsNeedsExceptionVariables()
 
     context_identifier = context.getContextObjectName()
-    frame_identifier = "frame_" + context_identifier
 
     context.addFrameDeclaration(
         template_frame_guard_cache_decl % {
-            "frame_identifier" : frame_identifier,
+            "frame_identifier" : "frame_" + context_identifier,
         }
     )
 
     no_exception_exit = context.allocateLabel("frame_no_exception")
 
+    frame_identifier = context_identifier + "->m_frame"
+
     emit(
         template_frame_guard_generator % {
             "context_identifier"     : context_identifier,
-            "frame_cache_identifier" : "cache_" + frame_identifier,
+            "frame_cache_identifier" : "cache_frame_" + context_identifier,
             "code_identifier"        : code_identifier,
             "locals_size"            : getFrameLocalsStorageSize(type_descriptions),
             "codes"                  : indented(codes, 0),
-            "module_identifier"      : getModuleAccessCode(context = context),
+            "module_identifier"      : getModuleAccessCode(context),
             "no_exception_exit"      : no_exception_exit,
         }
     )
@@ -342,6 +377,7 @@ def getFrameGuardLightCode(code_identifier, codes, parent_exception_exit,
     if frame_return_exit is not None:
         emit(
             template_frame_guard_generator_return_handler % {
+                "context_identifier" : context_identifier,
                 "frame_identifier"   : "%s->m_frame" % context_identifier,
                 "return_exit"        : parent_return_exit,
                 "frame_return_exit"  : frame_return_exit,
@@ -349,23 +385,18 @@ def getFrameGuardLightCode(code_identifier, codes, parent_exception_exit,
         )
 
     if frame_exception_exit is not None:
-        frame_variable_codes = context.getFrameVariableCodeNames()
-
-        if frame_variable_codes:
-            frame_variable_codes = ',' + frame_variable_codes
-        else:
-            frame_variable_codes = ""
-
         emit(
             template_frame_guard_generator_exception_handler % {
+                "context_identifier"     : context_identifier,
                 "frame_identifier"       : "%s->m_frame" % context_identifier,
-                "frame_cache_identifier" : "cache_" + frame_identifier,
+                "frame_cache_identifier" : "cache_frame_" + context_identifier,
                 "tb_making"              : getTracebackMakingIdentifier(
                                                context     = context,
                                                lineno_name = "exception_lineno"
                                            ),
-                "type_description"       : "type_description" if context.needsFrameVariableTypeDescription() else '""',
-                "frame_variable_refs"    : frame_variable_codes,
+                "attach_locals"          : indented(
+                    getFrameAttachLocalsCode(context, frame_identifier)
+                ),
                 "frame_exception_exit"   : frame_exception_exit,
                 "parent_exception_exit"  : parent_exception_exit
             }
@@ -386,7 +417,9 @@ def generateFramePreserveExceptionCode(statement, emit, context):
     else:
         preserver_id = statement.getPreserverId()
 
-        if preserver_id == 0 and python_version < 300:
+        if preserver_id == 0:
+            assert False
+
             emit(
                 "PRESERVE_FRAME_EXCEPTION( %(frame_identifier)s );" % {
                     "frame_identifier" : context.getFrameHandle()
@@ -395,13 +428,14 @@ def generateFramePreserveExceptionCode(statement, emit, context):
         else:
             context.addExceptionPreserverVariables(preserver_id)
 
+            # TODO: Multiple thread state calls should be avoided.
             emit(
                 """\
-exception_preserved_type_%(preserver_id)d = PyThreadState_GET()->exc_type;
+exception_preserved_type_%(preserver_id)d = EXC_TYPE(PyThreadState_GET());
 Py_XINCREF( exception_preserved_type_%(preserver_id)d );
-exception_preserved_value_%(preserver_id)d = PyThreadState_GET()->exc_value;
+exception_preserved_value_%(preserver_id)d = EXC_VALUE(PyThreadState_GET());
 Py_XINCREF( exception_preserved_value_%(preserver_id)d );
-exception_preserved_tb_%(preserver_id)d = (PyTracebackObject *)PyThreadState_GET()->exc_traceback;
+exception_preserved_tb_%(preserver_id)d = (PyTracebackObject *)EXC_TRACEBACK(PyThreadState_GET());
 Py_XINCREF( exception_preserved_tb_%(preserver_id)d );
 """ % {
                     "preserver_id"  : preserver_id,
@@ -421,18 +455,11 @@ def generateFrameRestoreExceptionCode(statement, emit, context):
     else:
         preserver_id = statement.getPreserverId()
 
-        if preserver_id == 0  and python_version < 300:
-            emit(
-                "RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );" % {
-                    "frame_identifier" : context.getFrameHandle()
-                }
-            )
-        else:
-            emit(
-                """\
+        emit(
+            """\
 SET_CURRENT_EXCEPTION( exception_preserved_type_%(preserver_id)d, \
 exception_preserved_value_%(preserver_id)d, \
 exception_preserved_tb_%(preserver_id)d );""" % {
-                    "preserver_id" : preserver_id,
-                }
-            )
+                "preserver_id" : preserver_id,
+            }
+        )
