@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,13 +22,13 @@ Next variants and unpacking with related checks.
 
 from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
 from .ErrorCodes import (
     getErrorExitCode,
     getErrorExitReleaseCode,
     getFrameVariableTypeDescriptionCode,
     getReleaseCode
 )
-from .Helpers import generateChildExpressionsCode, generateExpressionCode
 from .Indentation import indented
 from .LineNumberCodes import getErrorLineNumberUpdateCode
 from .PythonAPICodes import generateCAPIObjectCode
@@ -52,14 +52,9 @@ def generateBuiltinNext1Code(to_name, expression, emit, context):
         )
     )
 
-    getReleaseCode(
-        release_name = value_name,
-        emit         = emit,
-        context      = context
-    )
-
     getErrorExitCode(
         check_name      = to_name,
+        release_name    = value_name,
         quick_exception = "StopIteration",
         emit            = emit,
         context         = context
@@ -113,41 +108,6 @@ def getBuiltinLoopBreakNextCode(to_name, value, emit, context):
     context.addCleanupTempName(to_name)
 
 
-def getUnpackNextCode(to_name, value, expected, count, emit, context):
-    if python_version < 350:
-        emit(
-            "%s = UNPACK_NEXT( %s, %s );" % (
-                to_name,
-                value,
-                count - 1
-            )
-        )
-    else:
-        emit(
-            "%s = UNPACK_NEXT( %s, %s, %s );" % (
-                to_name,
-                value,
-                count - 1,
-                expected
-            )
-        )
-
-    getErrorExitCode(
-        check_name      = to_name,
-        quick_exception = "StopIteration",
-        emit            = emit,
-        context         = context
-    )
-
-    getReleaseCode(
-        release_name = value,
-        emit         = emit,
-        context      = context
-    )
-
-    context.addCleanupTempName(to_name)
-
-
 def generateSpecialUnpackCode(to_name, expression, emit, context):
     value_name = context.allocateTempName("unpack")
 
@@ -158,15 +118,40 @@ def generateSpecialUnpackCode(to_name, expression, emit, context):
         context    = context
     )
 
-    getUnpackNextCode(
-        to_name  = to_name,
-        value    = value_name,
-        count    = expression.getCount(),
-        expected = expression.getExpected(),
-        emit     = emit,
-        context  = context
+    count = expression.getCount()
+
+    if python_version < 350:
+        emit(
+            "%s = UNPACK_NEXT( %s, %s );" % (
+                to_name,
+                value_name,
+                count - 1
+            )
+        )
+    else:
+        starred = expression.getStarred()
+        expected = expression.getExpected()
+
+        emit(
+            "%s = UNPACK_NEXT%s( %s, %s, %s );" % (
+                to_name,
+                "_STARRED" if starred else "",
+                value_name,
+                count - 1,
+                expected
+            )
+        )
+
+    getErrorExitCode(
+        check_name      = to_name,
+        release_name    = value_name,
+        quick_exception = "StopIteration",
+        emit            = emit,
+        context         = context
     )
 
+    context.addCleanupTempName(to_name)
+
 
 def generateUnpackCheckCode(statement, emit, context):
     iterator_name  = context.allocateTempName("iterator_name")
@@ -243,6 +228,20 @@ def generateBuiltinIter1Code(to_name, expression, emit, context):
     )
 
 
+def generateBuiltinIterForUnpackCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "MAKE_UNPACK_ITERATOR",
+        arg_desc   = (
+            ("iter_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
+
+
 def generateBuiltinIter2Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name    = to_name,
