@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,6 +24,7 @@ very welcome.
 
 from __future__ import print_function
 
+import contextlib
 import hashlib
 import marshal
 import os
@@ -57,6 +58,7 @@ from nuitka.utils.FileOperations import (
     listDir,
     makePath
 )
+from nuitka.utils.ThreadedExecutor import Lock, ThreadPoolExecutor, waitWorkers
 from nuitka.utils.Timing import TimerReport
 
 from .DependsExe import getDependsExePath
@@ -695,28 +697,6 @@ def _detectBinaryPathDLLsMacOS(original_dir, binary_filename):
     return result
 
 
-def _makeBinaryPathPathDLLSearchEnv(package_name):
-    # Put the PYTHONPATH into the system "PATH", DLLs frequently live in
-    # the package directories.
-    env = os.environ.copy()
-    path = env.get("PATH","").split(os.pathsep)
-
-    # Put the "Python.exe" first. At least for WinPython, they put the DLLs
-    # there.
-    path = [sys.prefix] + sys.path + path
-
-    if package_name is not None:
-        for element in sys.path:
-            candidate = os.path.join(element, package_name)
-
-            if os.path.isdir(candidate):
-                path.append(candidate)
-
-
-    env["PATH"] = os.pathsep.join(path)
-
-    return env
-
 def _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filename):
     original_filename = os.path.join(
         original_dir,
@@ -753,70 +733,27 @@ def _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filen
     )
 
 
-def _detectBinaryPathDLLsWindows(is_main_executable, source_dir, original_dir, binary_filename,
-                                 package_name):
-    # This is complex, as it also includes the caching mechanism
-    # pylint: disable=too-many-branches
+# Locking seems to be only required for Windows currently, expressed that in the
+# lock name.
+windows_lock = None
 
-    result = set()
+@contextlib.contextmanager
+def _withLock():
+    # This is a singleton, pylint: disable=global-statement
+    global windows_lock
 
-    cache_filename = _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filename)
+    if windows_lock is None:
+        windows_lock = Lock()
 
-    if os.path.exists(cache_filename):
-        for line in open(cache_filename):
-            line = line.strip()
-
-            result.add(line)
-
-        return result
-
-    depends_exe = getDependsExePath()
-
-    # The search order by default prefers the system directory, where a
-    # wrong "PythonXX.dll" might be living.
-    with open(binary_filename + ".dwp", 'w') as dwp_file:
-        dwp_file.write("""\
-KnownDLLs
-SysPath
-AppDir
-{original_dirs}
-32BitSysDir
-16BitSysDir
-OSDir
-AppPath
-SxS
-""".format(
-            original_dirs = (
-                '\n'.join(
-                    "UserDir %s" % dirname
-                    for dirname in
-                    [original_dir] + getSubDirectories(original_dir)
-                    if not os.path.basename(dirname) == "__pycache__"
-                    if any(entry[1].lower().endswith(".dll") for entry in listDir(dirname))
-                )
-                if original_dir is not None
-                else ""
-            )
-        )
-    )
-
-    subprocess.call(
-        (
-            depends_exe,
-            "-c",
-            "-ot%s" % binary_filename + ".depends",
-            "-d:%s" % binary_filename + ".dwp",
-            "-f1",
-            "-pa1",
-            "-ps1",
-            binary_filename
-        ),
-        env = _makeBinaryPathPathDLLSearchEnv(package_name),
-    )
+    windows_lock.acquire()
+    yield
+    windows_lock.release()
 
+def _parseDependsExeOutput(filename, result):
     inside = False
     first = False
-    for line in open(binary_filename + ".depends"):
+
+    for line in open(filename):
         if "| Module Dependency Tree |" in line:
             inside = True
             first = True
@@ -904,19 +841,103 @@ SxS
             "FIREWALLAPI.DLL", "FLTLIB.DLL", "MRMCORER.DLL", "NTASN1.DLL",
             "SECHOST.DLL", "SETTINGSYNCPOLICY.DLL", "SHCORE.DLL", "TBS.DLL",
             "TWINAPI.APPCORE.DLL", "TWINAPI.DLL", "VIRTDISK.DLL",
-            "WEBSOCKET.DLL", "WEVTAPI.DLL", "WINMMBASE.DLL", "WMICLNT.DLL"):
+            "WEBSOCKET.DLL", "WEVTAPI.DLL", "WINMMBASE.DLL", "WMICLNT.DLL",
+            "ICUUC.DLL"):
             continue
 
         result.add(
             os.path.normcase(os.path.abspath(dll_filename))
         )
 
+
+def _detectBinaryPathDLLsWindows(is_main_executable, source_dir, original_dir, binary_filename, package_name):
+    # This is complex, as it also includes the caching mechanism
+    # pylint: disable=too-many-locals
+
+    result = set()
+
+    cache_filename = _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filename)
+
+    if os.path.exists(cache_filename) and not Options.shallNotUseDependsExeCachedResults():
+        for line in open(cache_filename):
+            line = line.strip()
+
+            result.add(line)
+
+        return result
+
+    # User query should only happen once if at all.
+    with _withLock():
+        depends_exe = getDependsExePath()
+
+    scan_dirs = [sys.prefix]
+
+    if package_name is not None:
+        from nuitka.importing.Importing import findModule
+
+        package_dir = findModule(None, package_name, None, 0, False)[1]
+
+        if os.path.isdir(package_dir):
+            scan_dirs.append(package_dir)
+            scan_dirs.extend(getSubDirectories(package_dir))
+
+    if original_dir is not None:
+        scan_dirs.append(original_dir)
+        scan_dirs.extend(getSubDirectories(original_dir))
+
+    with _withLock():
+        # The search order by default prefers the system directory, where a
+        # wrong "PythonXX.dll" might be living.
+        with open(binary_filename + ".dwp", 'w') as dwp_file:
+            dwp_file.write(
+                """\
+KnownDLLs
+%(original_dirs)s
+SysPath
+32BitSysDir
+16BitSysDir
+OSDir
+AppPath
+SxS
+""" % {
+                "original_dirs" : '\n'.join(
+                    "UserDir %s" % dirname
+                    for dirname in
+                    scan_dirs
+                    if not os.path.basename(dirname) == "__pycache__"
+                    if any(entry[1].lower().endswith(".dll") for entry in listDir(dirname))
+                ),
+                }
+            )
+
+        # Starting the process while locked, so file handles are not duplicated.
+        depends_exe_process = subprocess.Popen(
+            (
+                depends_exe,
+                "-c",
+                "-ot%s" % binary_filename + ".depends",
+                "-d:%s" % binary_filename + ".dwp",
+                "-f1",
+                "-pa1",
+                "-ps1",
+                binary_filename
+            )
+        )
+
+    # TODO: Exit code should be checked.
+    depends_exe_process.wait()
+
+    # Opening the result under lock, so it is not getting locked by new processes.
+    with _withLock():
+        _parseDependsExeOutput(binary_filename + ".depends", result)
+
     deleteFile(binary_filename + ".depends", must_exist = True)
     deleteFile(binary_filename + ".dwp", must_exist = True)
 
-    with open(cache_filename, 'w') as cache_file:
-        for dll_filename in result:
-            print(dll_filename, file = cache_file)
+    if not Options.shallNotStoreDependsExeCachedResults():
+        with open(cache_filename, 'w') as cache_file:
+            for dll_filename in result:
+                print(dll_filename, file = cache_file)
 
     return result
 
@@ -954,26 +975,39 @@ def detectBinaryDLLs(is_main_executable, source_dir, original_filename,
 
 
 def detectUsedDLLs(source_dir, standalone_entry_points):
-    result = OrderedDict()
-
-    for count, (original_filename, binary_filename, package_name) in enumerate(standalone_entry_points):
+    def addDLLInfo(count, source_dir, original_filename, binary_filename, package_name):
         used_dlls = detectBinaryDLLs(
             is_main_executable = count == 0,
             source_dir         = source_dir,
             original_filename  = original_filename,
             binary_filename    = binary_filename,
-            package_name       = package_name,
+            package_name       = package_name
         )
 
-        for dll_filename in used_dlls:
-            # We want these to be absolute paths. Solve that in the parts
-            # where detectBinaryDLLs is platform specific.
-            assert os.path.isabs(dll_filename), dll_filename
+        return binary_filename, used_dlls
 
-            if dll_filename not in result:
-                result[dll_filename] = []
+    result = OrderedDict()
 
-            result[dll_filename].append(binary_filename)
+    with ThreadPoolExecutor(max_workers = Utils.getCoreCount() * 3) as worker_pool:
+        workers = []
+
+        for count, (original_filename, binary_filename, package_name) in enumerate(standalone_entry_points):
+            workers.append(
+                worker_pool.submit(
+                    addDLLInfo,
+                    count, source_dir, original_filename, binary_filename, package_name
+                )
+            )
+
+        for binary_filename, used_dlls in waitWorkers(workers):
+            for dll_filename in used_dlls:
+                # We want these to be absolute paths. Solve that in the parts
+                # where detectBinaryDLLs is platform specific.
+                assert os.path.isabs(dll_filename), dll_filename
+
+                if dll_filename not in result:
+                    result[dll_filename] = []
+                result[dll_filename].append(binary_filename)
 
     return result
 
@@ -1074,12 +1108,20 @@ def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
 
     used_dlls = detectUsedDLLs(source_dir, standalone_entry_points)
 
+    removed_dlls = set()
+
     # Fist make checks and remove some.
     for dll_filename1, sources1 in tuple(iterItems(used_dlls)):
+        if dll_filename1 in removed_dlls:
+            continue
+
         for dll_filename2, sources2 in tuple(iterItems(used_dlls)):
             if dll_filename1 == dll_filename2:
                 continue
 
+            if dll_filename2 in removed_dlls:
+                continue
+
             # Colliding basenames are an issue to us.
             if os.path.basename(dll_filename1) != \
                os.path.basename(dll_filename2):
@@ -1109,6 +1151,8 @@ def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
             import filecmp
             if filecmp.cmp(dll_filename1, dll_filename2):
                 del used_dlls[dll_filename2]
+                removed_dlls.add(dll_filename2)
+
                 continue
 
             # So we have conflicting DLLs, in which case we do not proceed.
