@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,13 +24,9 @@ whose implementation lives here. The creation itself also lives here.
 
 from nuitka.PythonVersions import python_version
 
-from .Checkers import checkStatementsSequenceOrNone
 from .ExpressionBases import ExpressionChildrenHavingBase
 from .FunctionNodes import ExpressionFunctionEntryPointBase
-from .IndicatorMixins import (
-    MarkLocalsDictIndicatorMixin,
-    MarkUnoptimizedFunctionIndicatorMixin
-)
+from .IndicatorMixins import MarkUnoptimizedFunctionIndicatorMixin
 from .ReturnNodes import StatementReturn, StatementReturnNone
 
 
@@ -44,7 +40,7 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
     getGeneratorRef = ExpressionChildrenHavingBase.childGetter("generator_ref")
 
     def __init__(self, generator_ref, code_object, source_ref):
-        assert generator_ref.getFunctionBody().isExpressionGeneratorObjectBody()
+        assert generator_ref.getFunctionBody().isExpressionGeneratorObjectBody(), generator_ref
 
         ExpressionChildrenHavingBase.__init__(
             self,
@@ -73,7 +69,9 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, trace)
+            )
 
         # TODO: Generator body may know something too.
         return self, None, None
@@ -85,29 +83,13 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
         return False
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
 
 
-
-class ExpressionGeneratorObjectBody(MarkLocalsDictIndicatorMixin,
-                                    MarkUnoptimizedFunctionIndicatorMixin,
+class ExpressionGeneratorObjectBody(MarkUnoptimizedFunctionIndicatorMixin,
                                     ExpressionFunctionEntryPointBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, pylint: disable=R0901
     kind = "EXPRESSION_GENERATOR_OBJECT_BODY"
 
-    named_children = (
-        "body",
-    )
-
-    checkers = {
-        # TODO: Is "None" really an allowed value.
-        "body" : checkStatementsSequenceOrNone
-    }
-
     if python_version >= 340:
         qualname_setup = None
 
@@ -121,8 +103,6 @@ class ExpressionGeneratorObjectBody(MarkLocalsDictIndicatorMixin,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicatorMixin.__init__(self)
-
         MarkUnoptimizedFunctionIndicatorMixin.__init__(self, flags)
 
         self.needs_generator_return_exit = False
@@ -193,6 +173,9 @@ Generator return value is always None."""
     def isStatementGeneratorReturn():
         return True
 
+    def getStatementNiceName(self):
+        return "generator return statement"
+
 
 class StatementGeneratorReturnNone(StatementReturnNone):
     kind = "STATEMENT_GENERATOR_RETURN_NONE"
@@ -208,3 +191,6 @@ class StatementGeneratorReturnNone(StatementReturnNone):
     @staticmethod
     def isStatementGeneratorReturn():
         return True
+
+    def getStatementNiceName(self):
+        return "generator return statement"
