@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,8 +29,7 @@ from nuitka.PythonVersions import python_version
 from .ConstantRefNodes import makeConstantRefNode
 from .DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
 from .ExpressionBases import ExpressionBase, ExpressionBuiltinSingleArgBase
-from .NodeBases import StatementChildrenHavingBase
-from .VariableRefNodes import ExpressionVariableRef
+from .VariableRefNodes import ExpressionTempVariableRef, ExpressionVariableRef
 
 
 class ExpressionBuiltinGlobals(ExpressionBase):
@@ -42,6 +41,9 @@ class ExpressionBuiltinGlobals(ExpressionBase):
             source_ref = source_ref
         )
 
+    def finalize(self):
+        del self.parent
+
     def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
@@ -51,22 +53,24 @@ class ExpressionBuiltinGlobals(ExpressionBase):
     def mayRaiseException(self, exception_type):
         return False
 
-    def mayBeNone(self):
-        return False
-
 
 class ExpressionBuiltinLocalsBase(ExpressionBase):
     # Base classes can be abstract, pylint: disable=abstract-method
 
-    __slots__ = ("variable_versions",)
+    __slots__ = ("variable_traces", "locals_scope")
 
-    def __init__(self, source_ref):
+    def __init__(self, locals_scope, source_ref):
         ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-        self.variable_versions = None
+        self.variable_traces = None
+        self.locals_scope = locals_scope
+
+    def finalize(self):
+        del self.locals_scope
+        del self.variable_traces
 
     def mayHaveSideEffects(self):
         if python_version < 300:
@@ -82,29 +86,80 @@ class ExpressionBuiltinLocalsBase(ExpressionBase):
     def mayRaiseException(self, exception_type):
         return self.mayHaveSideEffects()
 
-    def mayBeNone(self):
-        return False
-
-    def getVariableVersions(self):
-        return self.variable_versions
+    def getVariableTraces(self):
+        return self.variable_traces
 
 
 class ExpressionBuiltinLocalsUpdated(ExpressionBuiltinLocalsBase):
     kind = "EXPRESSION_BUILTIN_LOCALS_UPDATED"
 
-    def needsLocalsDict(self):
-        return not self.getParent().isStatementReturn()
+    def __init__(self, locals_scope, source_ref):
+        ExpressionBuiltinLocalsBase.__init__(
+            self,
+            locals_scope = locals_scope,
+            source_ref   = source_ref
+        )
+
+        assert locals_scope is not None
+
+    def getLocalsScope(self):
+        return self.locals_scope
 
     def computeExpressionRaw(self, trace_collection):
         # Just inform the collection that all escaped.
-        self.variable_versions = trace_collection.onLocalsUsage(self.getParentVariableProvider())
+        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
-        if self.getParent().isStatementReturn():
-            result = ExpressionBuiltinLocalsCopy(
-                source_ref = self.getSourceReference()
+        trace_collection.onLocalsDictEscaped(self.locals_scope)
+
+        return self, None, None
+
+
+class ExpressionBuiltinLocalsRef(ExpressionBuiltinLocalsBase):
+    kind = "EXPRESSION_BUILTIN_LOCALS_REF"
+
+    def __init__(self, locals_scope, source_ref):
+        ExpressionBuiltinLocalsBase.__init__(
+            self,
+            locals_scope = locals_scope,
+            source_ref   = source_ref
+        )
+
+    def getLocalsScope(self):
+        return self.locals_scope
+
+    def computeExpressionRaw(self, trace_collection):
+        if self.locals_scope.isMarkedForPropagation():
+            result = ExpressionMakeDict(
+                pairs      = (
+                    ExpressionKeyValuePair(
+                        key        = makeConstantRefNode(
+                            constant   = variable_name,
+                            source_ref = self.source_ref
+                        ),
+                        value      = ExpressionTempVariableRef(
+                            variable   = variable,
+                            source_ref = self.source_ref
+                        ),
+                        source_ref = self.source_ref
+                    )
+                    for variable_name, variable in
+                    self.locals_scope.getPropagationVariables().items()
+                ),
+                source_ref = self.source_ref
             )
 
-            return result, "new_expression", "Locals does not escape, no write back needed."
+            new_result = result.computeExpressionRaw(trace_collection)
+
+            assert new_result[0] is result
+
+            self.finalize()
+
+            return result, "new_expression", "Propagated locals dictionary reference."
+
+
+        # Just inform the collection that all escaped unless it is abortative.
+        if not self.getParent().isStatementReturn():
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return self, None, None
 
@@ -112,28 +167,22 @@ class ExpressionBuiltinLocalsUpdated(ExpressionBuiltinLocalsBase):
 class ExpressionBuiltinLocalsCopy(ExpressionBuiltinLocalsBase):
     kind = "EXPRESSION_BUILTIN_LOCALS_COPY"
 
-    def needsLocalsDict(self):
-        return False
-
     def computeExpressionRaw(self, trace_collection):
         # Just inform the collection that all escaped.
-        self.variable_versions = trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
-        for variable, version in self.variable_versions:
-            trace = trace_collection.getVariableTrace(variable, version)
+        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
-            if not trace.mustHaveValue() and not trace.mustNotHaveValue():
+        for variable, variable_trace in self.variable_traces:
+            if not variable_trace.mustHaveValue() and not variable_trace.mustNotHaveValue():
                 return self, None, None
 
             # Other locals elsewhere.
-            if trace.getNameUsageCount() > 1:
+            if variable_trace.getNameUsageCount() > 1:
                 return self, None, None
 
         pairs = []
-        for variable, version in self.variable_versions:
-            trace = trace_collection.getVariableTrace(variable, version)
-
-            if trace.mustHaveValue():
+        for variable, variable_trace in self.variable_traces:
+            if variable_trace.mustHaveValue():
                 pairs.append(
                     ExpressionKeyValuePair(
                         key        = makeConstantRefNode(
@@ -168,59 +217,9 @@ class ExpressionBuiltinLocalsCopy(ExpressionBuiltinLocalsBase):
         return result, "new_expression", "Statically predicted locals dictionary."
 
 
-class StatementSetLocals(StatementChildrenHavingBase):
-    kind = "STATEMENT_SET_LOCALS"
-
-    named_children = (
-        "new_locals",
-    )
-
-    def __init__(self, new_locals, source_ref):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "new_locals" : new_locals,
-            },
-            source_ref = source_ref
-        )
-
-    def needsLocalsDict(self):
-        return True
-
-    def mayRaiseException(self, exception_type):
-        return self.getNewLocals().mayRaiseException(exception_type)
-
-    getNewLocals = StatementChildrenHavingBase.childGetter("new_locals")
-
-    def computeStatement(self, trace_collection):
-        # Make sure that we don't even assume "unset" of things not set yet for
-        # anything.
-        trace_collection.removeAllKnowledge()
-
-        trace_collection.onExpression(self.getNewLocals())
-        new_locals = self.getNewLocals()
-
-        if new_locals.willRaiseException(BaseException):
-            from .NodeMakingHelpers import \
-               makeStatementExpressionOnlyReplacementNode
-
-            result = makeStatementExpressionOnlyReplacementNode(
-                expression = new_locals,
-                node       = self
-            )
-
-            return result, "new_raise", """\
-Setting locals already raises implicitly building new locals."""
-
-        return self, None, None
-
-
 class ExpressionBuiltinDir1(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_BUILTIN_DIR1"
 
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
         return self, None, None
-
-    def mayBeNone(self):
-        return False
