@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,7 +30,7 @@ import tempfile
 from contextlib import contextmanager
 
 from nuitka.Tracing import my_print
-from nuitka.utils.AppDirs import getCacheDir
+from nuitka.utils.AppDirs import getAppDir, getCacheDir
 from nuitka.utils.Execution import check_output
 from nuitka.utils.FileOperations import makePath, removeDirectory
 
@@ -60,11 +60,7 @@ def check_result(*popenargs, **kwargs):
         return True
 
 
-_python_version = None
-_python_arch = None
-_python_executable = None
-
-def setup(needs_io_encoding = False, silent = False):
+def goMainDir():
     # Go its own directory, to have it easy with path knowledge.
     os.chdir(
         os.path.dirname(
@@ -72,6 +68,13 @@ def setup(needs_io_encoding = False, silent = False):
         )
     )
 
+_python_version = None
+_python_arch = None
+_python_executable = None
+
+def setup(needs_io_encoding = False, silent = False):
+    goMainDir()
+
     if "PYTHON" not in os.environ:
         os.environ["PYTHON"] = sys.executable
 
@@ -184,21 +187,31 @@ def convertUsing2to3(path, force = False):
     # This may already be a temp file, e.g. because of construct creation.
     try:
         shutil.copy(path, new_path)
-    except shutil.SameFileError:  # @UndefinedVariable
+    except shutil.Error:
         pass
 
-    # On Windows, we cannot rely on 2to3 to be in the path.
-    if os.name == "nt":
-        command = [
-            sys.executable,
-            os.path.join(
-                os.path.dirname(sys.executable),
-                "Tools/Scripts/2to3.py"
-            )
-        ]
+    # For Python2.6 and 3.2 the -m lib2to3 was not yet supported.
+    use_binary = sys.version_info[:2] in ((2,6), (3,2))
+
+    if use_binary:
+        # On Windows, we cannot rely on 2to3 to be in the path.
+        if os.name == "nt":
+            command = [
+                sys.executable,
+                os.path.join(
+                    os.path.dirname(sys.executable),
+                    "Tools/Scripts/2to3.py"
+                )
+            ]
+        else:
+            command = [
+                "2to3"
+            ]
     else:
         command = [
-            "2to3"
+            sys.executable,
+            "-m",
+            "lib2to3",
         ]
 
     command += [
@@ -434,7 +447,7 @@ def getDependsExePath():
     if "APPDATA" not in os.environ:
         sys.exit("Error, standalone mode cannot find 'APPDATA' environment.")
 
-    nuitka_app_dir = os.path.join(os.environ["APPDATA"],"nuitka")
+    nuitka_app_dir = getAppDir()
 
     depends_dir = os.path.join(
         nuitka_app_dir,
@@ -541,6 +554,13 @@ Error, needs 'strace' on your system to scan used libraries."""
                line.startswith(b"readlink("):
                 filename = line[line.find(b"(")+2:line.find(b", ")-1]
 
+                # At least Python3.7 considers the default Python3 path.
+                if filename == b"/usr/bin/python3":
+                    continue
+
+                if filename in (b"/usr/bin/python3." + version for version in (b"5", b"6", b"7")):
+                    continue
+
                 binary_path = _python_executable
                 if str is not bytes:
                     binary_path = binary_path.encode("utf-8")
@@ -626,6 +646,9 @@ Error, needs 'strace' on your system to scan used libraries."""
 
 
 def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
+    # A lot of special white listing is required.
+    # pylint: disable=too-many-branches,too-many-statements
+
     result = []
 
     for loaded_filename in loaded_filenames:
@@ -714,6 +737,10 @@ def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
         if loaded_basename.startswith("libicu"):
             continue
 
+        # GTK may access X files.
+        if loaded_basename == ".Xauthority":
+            continue
+
         result.append(loaded_filename)
 
     return result
@@ -804,10 +831,10 @@ def checkReferenceCount(checked_function, max_rounds = 10):
             for key in m1:
                 if key not in m2:
                     print('*' * 80)
-                    print(key)
+                    print("extra", key)
                 elif m1[key] != m2[key]:
                     print('*' * 80)
-                    print(key)
+                    print(m1[key], "->", m2[key], key)
                 else:
                     pass
                     # print m1[key]
@@ -903,14 +930,36 @@ def checkDebugPython():
     if not hasattr(sys, "gettotalrefcount"):
         my_print("Warning, using non-debug Python makes this test ineffective.")
         sys.gettotalrefcount = lambda : 0
+    elif sys.version_info >= (3,7,0) and sys.version_info < (3,7,1):
+        my_print("Warning, bug of CPython 3.7.0 breaks reference counting and makes this test ineffective.")
+        sys.gettotalrefcount = lambda : 0
 
 
-@contextmanager
-def withPythonPathChange(python_path):
+def addToPythonPath(python_path):
     if type(python_path) in (tuple, list):
         python_path = os.pathsep.join(python_path)
 
     if python_path:
+        if "PYTHONPATH" in os.environ:
+            os.environ["PYTHONPATH"] += os.pathsep + python_path
+        else:
+            os.environ["PYTHONPATH"] = python_path
+
+
+@contextmanager
+def withPythonPathChange(python_path):
+    if python_path:
+        if type(python_path) not in (tuple, list):
+            python_path = python_path.split(os.pathsep)
+
+        python_path = [
+            os.path.normpath(os.path.abspath(element))
+            for element in
+            python_path
+        ]
+
+        python_path = os.pathsep.join(python_path)
+
         if "PYTHONPATH" in os.environ:
             old_path = os.environ["PYTHONPATH"]
             os.environ["PYTHONPATH"] += os.pathsep + python_path
@@ -918,6 +967,13 @@ def withPythonPathChange(python_path):
             old_path = None
             os.environ["PYTHONPATH"] = python_path
 
+#     print(
+#         "Effective PYTHONPATH in %s is %r" % (
+#             sys.modules["__main__"],
+#             os.environ.get("PYTHONPATH", "")
+#         )
+#     )
+
     yield
 
     if python_path:
@@ -1168,3 +1224,25 @@ def getTestingCacheDir():
     result = os.path.join(cache_dir, "tests_state")
     makePath(result)
     return result
+
+
+@contextmanager
+def withDirectoryChange(path, allow_none = False):
+    if path is not None or not allow_none:
+        old_cwd = os.getcwd()
+        os.chdir(path)
+
+    yield
+
+    if path is not None or not allow_none:
+        os.chdir(old_cwd)
+
+
+def someGenerator():
+    yield 1
+    yield 2
+    yield 3
+
+def someGeneratorRaising():
+    yield 1
+    raise TypeError(2)
