@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,6 @@
 
 from __future__ import print_function
 
-import difflib
 import os
 import re
 import subprocess
@@ -31,48 +30,9 @@ import sys
 import tempfile
 import time
 
-ran_tests_re                 = re.compile(r"^(Ran \d+ tests? in )\-?\d+\.\d+s$")
-instance_re                  = re.compile(r"at (?:0x)?[0-9a-fA-F]+(;?\s|\>)")
-thread_re                    = re.compile(r"Thread 0x[0-9a-fA-F]+")
-compiled_types_re            = re.compile(r"compiled_(module|function|generator|method|frame|coroutine)")
-module_repr_re               = re.compile(r"(\<module '.*?' from ').*?('\>)")
-
-global_name_error_re         = re.compile(
-    r"global (name ')(.*?)(' is not defined)"
-)
-non_ascii_error_rt           = re.compile(
-    r"(SyntaxError: Non-ASCII character.*? on line) \d+"
-)
-python_win_lib_re            = re.compile(
-    r"[a-zA-Z]:\\\\?[Pp]ython(.*?\\\\?)[Ll]ib"
-)
-local_port_re = re.compile(
-    r"(127\.0\.0\.1):\d{2,5}"
-)
-
-
-traceback_re                 = re.compile(
-    r'(F|f)ile "(.*?)", line (\d+)'
-)
-
-tempfile_re                  = re.compile(
-    r'/tmp/tmp[a-z0-9_]*'
-)
-
-def traceback_re_callback(match):
-    return r'%sile "%s", line %s' % (
-        match.group(1),
-        os.path.realpath(os.path.abspath(match.group(2))),
-        match.group(3)
-    )
-
-
-# Make sure we flush after every print, the "-u" option does more than that
-# and this is easy enough.
-def my_print(*args, **kwargs):
-    print(*args, **kwargs)
-
-    sys.stdout.flush()
+from nuitka.tools.testing.Common import addToPythonPath, withPythonPathChange
+from nuitka.tools.testing.OutputComparison import compareOutput
+from nuitka.Tracing import my_print
 
 
 def displayOutput(stdout, stderr):
@@ -85,145 +45,23 @@ def displayOutput(stdout, stderr):
         my_print(stderr)
 
 
-def compareOutput(kind, out_cpython, out_nuitka, ignore_warnings, ignore_infos,
-                  syntax_errors):
-    fromdate = ""
-    todate = ""
-
-    diff = difflib.unified_diff(
-        makeDiffable(out_cpython, ignore_warnings, ignore_infos, syntax_errors),
-        makeDiffable(out_nuitka, ignore_warnings, ignore_infos, syntax_errors),
-        "{program} ({detail})".format(
-            program = os.environ["PYTHON"],
-            detail  = kind
-        ),
-        "{program} ({detail})".format(
-            program = "nuitka",
-            detail  = kind
-        ),
-        fromdate,
-        todate,
-        n = 3
-    )
-
-    result = list(diff)
-
-    if result:
-        for line in result:
-            my_print(line, end = '\n' if not line.startswith("---") else "")
-
-        return 1
-    else:
-        return 0
 
 
-def makeDiffable(output, ignore_warnings, ignore_infos, syntax_errors):
-    # Of course many cases to deal with, pylint: disable=too-many-branches
 
-    result = []
+def checkNoPermissionError(output):
+    # Forms of permission errors.
+    for candidate in (b"Permission denied:",
+                      b"PermissionError:",
+                      b"DBPermissionsError:"):
+        if candidate in output:
+            return False
 
-    # Fix import "readline" because output sometimes starts with "\x1b[?1034h"
-    m = re.match(b'\\x1b\\[[^h]+h', output)
-    if m:
-        output = output[len(m.group()):]
+    # These are localized it seems.
+    if re.search(b"(WindowsError|FileNotFoundError|FileExistsError|WinError 145):"
+                 b".*(@test|totest|xx|Error 145)", output):
+        return False
 
-    lines = output.split(b"\n")
-    if syntax_errors:
-
-        for line in lines:
-            if line.startswith(b"SyntaxError:"):
-                lines = [line]
-                break
-
-    for line in lines:
-        if type(line) is not str:
-            line = line.decode("utf-8" if os.name != "nt" else "cp850")
-
-        if line.endswith('\r'):
-            line = line[:-1]
-
-        if line.startswith("REFCOUNTS"):
-            first_value = line[line.find('[')+1:line.find(',')]
-            last_value = line[line.rfind(' ')+1:line.rfind(']')]
-            line = line.\
-              replace(first_value, "xxxxx").\
-              replace(last_value, "xxxxx")
-
-        if line.startswith('[') and line.endswith("refs]"):
-            continue
-
-        if ignore_warnings and line.startswith("Nuitka:WARNING"):
-            continue
-
-        if ignore_infos and line.startswith("Nuitka:INFO"):
-            continue
-
-        if line.startswith("Nuitka:WARNING:Cannot recurse to import"):
-            continue
-
-        line = instance_re.sub(r"at 0xxxxxxxxx\1", line)
-        line = thread_re.sub(r"Thread 0xXXXXXXXX", line)
-        line = compiled_types_re.sub(r"\1", line)
-        line = global_name_error_re.sub(r"\1\2\3", line)
-        line = module_repr_re.sub(r"\1xxxxx\2", line)
-        line = non_ascii_error_rt.sub(r"\1 xxxx", line)
-
-        # Windows has a different "os.path", update according to it.
-        line = line.replace("ntpath", "posixpath")
-
-        # This URL is updated, and Nuitka outputs the new one, but we test
-        # against versions that don't have that.
-        line = line.replace(
-            "http://www.python.org/peps/pep-0263.html",
-            "http://python.org/dev/peps/pep-0263/",
-        )
-
-        line = ran_tests_re.sub(r"\1x.xxxs", line)
-
-        line = traceback_re.sub(traceback_re_callback, line)
-
-        line = tempfile_re.sub(r"/tmp/tmpxxxxxxx", line)
-
-        # This is a bug potentially, occurs only for CPython when re-directed,
-        # we are going to ignore the issue as Nuitka is fine.
-        if line == """\
-Exception RuntimeError: 'maximum recursion depth \
-exceeded while calling a Python object' in \
-<type 'exceptions.AttributeError'> ignored""":
-            continue
-
-        # This is also a bug potentially, but only visible under
-        # CPython
-        line = python_win_lib_re.sub(r"C:\\Python\1Lib", line)
-
-        # Port numbers can be random, lets ignore them
-        line = local_port_re.sub(r"\1:xxxxx", line)
-
-        # This is a bug with clang potentially, can't find out why it says that.
-        if line == "/usr/bin/ld: warning: .init_array section has zero size":
-            continue
-
-        # This is for NetBSD and OpenBSD, which seems to build "libpython" so
-        # that it gives such warnings.
-        if "() possibly used unsafely" in line or \
-           "() is almost always misused" in line:
-            continue
-
-        # This is for CentOS5, where the linker says this, and it's hard to
-        # disable
-        if "skipping incompatible /usr/lib/libpython2.6.so" in line:
-            continue
-
-        # This is for self compiled Python with default options, gives this
-        # harmless option for every time we link to "libpython".
-        if "is dangerous, better use `mkstemp'" in line or \
-           "In function `posix_tempnam'" in line or \
-           "In function `posix_tmpnam'" in line:
-            continue
-
-        result.append(line)
-
-    return result
+    return True
 
 
 def main():
@@ -254,7 +92,7 @@ def main():
     two_step_execution = hasArg("two_step_execution")
     binary_python_path = hasArg("binary_python_path")
     keep_python_path   = hasArg("keep_python_path")
-    trace_command      = hasArg("trace_command")
+    trace_command      = hasArg("trace_command") or os.environ.get("NUITKA_TRACE_COMMANDS", '0') != '0'
     remove_output      = hasArg("remove_output")
     standalone_mode    = hasArg("standalone")
     no_site            = hasArg("no_site")
@@ -409,17 +247,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         if comparison_mode:
             nuitka_call = [
                 os.environ["PYTHON"],
-                os.path.abspath(
-                    os.path.join(
-                        os.path.dirname(__file__),
-                        "..",
-                        "..",
-                        "..",
-                        "..",
-                        "bin",
-                        "nuitka"
-                    )
-                )
+                "-m",
+                "nuitka.__main__", # Note: Needed for Python2.6
             ]
         else:
             assert coverage_mode
@@ -433,17 +262,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
                 "--rcfile",
                 os.devnull,
                 "-a",
-                os.path.abspath(
-                    os.path.join(
-                        os.path.dirname(__file__),
-                        "..",
-                        "..",
-                        "..",
-                        "..",
-                        "bin",
-                        "nuitka"
-                    )
-                )
+                "-m",
+                "nuitka.__main__" # Note: Needed for Python2.6
             ]
 
     if python_debug:
@@ -478,11 +298,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
         os.environ["PYTHONPATH"] = python_path.strip()
 
     if binary_python_path:
-        python_path = os.environ.get("PYTHONPATH", "")
-        os.environ["PYTHONPATH"] = os.pathsep.join(
-            python_path.split(os.pathsep) + \
-            [os.path.dirname(os.path.abspath(filename))]
-        )
+        addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
     if keep_python_path or binary_python_path:
         extra_options.append("--keep-pythonpath")
@@ -573,18 +389,28 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
     if comparison_mode:
         start_time = time.time()
 
-        process = subprocess.Popen(
-            args   = cpython_cmd,
-            stdout = subprocess.PIPE,
-            stderr = subprocess.PIPE
-        )
+        # Try a coupile of times for permission denied, on Windows it can
+        # be transient.
+        for _i in range(5):
+            with withPythonPathChange(os.getcwd()):
+                process = subprocess.Popen(
+                    args   = cpython_cmd,
+                    stdout = subprocess.PIPE,
+                    stderr = subprocess.PIPE
+                )
 
-        stdout_cpython, stderr_cpython = process.communicate()
-        exit_cpython = process.returncode
+            stdout_cpython, stderr_cpython = process.communicate()
+            exit_cpython = process.returncode
+
+            if checkNoPermissionError(stdout_cpython) and \
+               checkNoPermissionError(stderr_cpython):
+                break
+
+            my_print("Retrying CPython due to permission problems after delay.")
+            time.sleep(2)
 
         cpython_time = time.time() - start_time
 
-
     if comparison_mode and not silent_mode:
         displayOutput(stdout_cpython, stderr_cpython)
 
@@ -613,23 +439,37 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".
     if not two_step_execution:
         if trace_command:
             my_print("Nuitka command:", nuitka_cmd)
-        process = subprocess.Popen(
-            args   = nuitka_cmd,
-            stdout = subprocess.PIPE,
-            stderr = subprocess.PIPE
-        )
 
-        stdout_nuitka, stderr_nuitka = process.communicate()
-        exit_nuitka = process.returncode
+        # Try a couple of times for permission denied, on Windows it can
+        # be transient.
+        for _i in range(5):
+            with withPythonPathChange(nuitka_package_dir):
+                process = subprocess.Popen(
+                    args   = nuitka_cmd,
+                    stdout = subprocess.PIPE,
+                    stderr = subprocess.PIPE
+                )
+
+            stdout_nuitka, stderr_nuitka = process.communicate()
+            exit_nuitka = process.returncode
+
+            if checkNoPermissionError(stdout_nuitka) and \
+               checkNoPermissionError(stderr_nuitka):
+                break
+
+            my_print("Retrying nuitka exe due to permission problems after delay.")
+            time.sleep(2)
+
     else:
         if trace_command:
             my_print("Nuitka command 1:", nuitka_cmd1)
 
-        process = subprocess.Popen(
-            args   = nuitka_cmd1,
-            stdout = subprocess.PIPE,
-            stderr = subprocess.PIPE
-        )
+        with withPythonPathChange(nuitka_package_dir):
+            process = subprocess.Popen(
+                args   = nuitka_cmd1,
+                stdout = subprocess.PIPE,
+                stderr = subprocess.PIPE
+            )
 
         stdout_nuitka1, stderr_nuitka1 = process.communicate()
         exit_nuitka1 = process.returncode
@@ -717,13 +557,18 @@ Exit codes {exit_cpython:d} (CPython) != {exit_nuitka:d} (Nuitka)""".format(
 
         # In case of segfault, also output the call stack by entering debugger
         # without stdin forwarded.
-        if exit_code_return and exit_nuitka == -11 and sys.platform != "nt":
+        if exit_code_return and \
+           exit_nuitka == -11 and \
+           sys.platform != "nt" and \
+           not module_mode and \
+           not two_step_execution:
             nuitka_cmd.insert(len(nuitka_cmd) - 1, "--debugger")
 
-            process = subprocess.Popen(
-                args  = nuitka_cmd,
-                stdin = subprocess.PIPE
-            )
+            with withPythonPathChange(nuitka_package_dir):
+                process = subprocess.Popen(
+                    args  = nuitka_cmd,
+                    stdin = subprocess.PIPE
+                )
 
             process.communicate()
 
@@ -748,6 +593,9 @@ Exit codes {exit_cpython:d} (CPython) != {exit_nuitka:d} (Nuitka)""".format(
                     # It appears there is a tiny lock race that we randomly cause,
                     # likely because --run spawns a subprocess that might still
                     # be doing the cleanup work.
+                    if os.path.exists(nuitka_cmd2[0]+".away"):
+                        os.unlink(nuitka_cmd2[0]+".away")
+
                     for _i in range(10):
                         try:
                             os.rename(nuitka_cmd2[0], nuitka_cmd2[0]+".away")
@@ -764,8 +612,6 @@ Exit codes {exit_cpython:d} (CPython) != {exit_nuitka:d} (Nuitka)""".format(
                         else:
                             break
 
-                    assert not os.path.exists(nuitka_cmd2[0]+".away")
-
                     if os.path.exists(pdb_filename):
                         os.unlink(pdb_filename)
                 else:
@@ -792,23 +638,25 @@ Exit codes {exit_cpython:d} (CPython) != {exit_nuitka:d} (Nuitka)""".format(
         my_print("OK, same outputs.")
 
 
+nuitka_package_dir = os.path.normpath(
+    os.path.abspath(
+        os.path.join(
+            os.path.dirname(__file__),
+            "..",
+            "..",
+            "..",
+            "..",
+        )
+    )
+)
+
+
 if __name__ == "__main__":
     # Unchanged, running from checkout, use the parent directory, the nuitka
     # package ought be there.
     sys.path.insert(
         0,
-        os.path.normpath(
-            os.path.join(
-                os.path.dirname(__file__),
-                "..",
-                "..",
-                "..",
-            )
-        )
-    )
-    sys.path.insert(
-        1,
-        "/usr/share/nuitka"
+        nuitka_package_dir
     )
 
     main()
