@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,6 +31,7 @@ import tempfile
 from optparse import OptionParser
 
 from nuitka.tools.Basics import goHome
+from nuitka.tools.testing.Common import withDirectoryChange
 from nuitka.utils.Execution import (
     check_output,
     getExecutablePath,
@@ -38,15 +39,9 @@ from nuitka.utils.Execution import (
 )
 
 
-def main():
-    # There are freaking many options to honor, pylint: disable=too-many-branches
-
-    # Lets honor this Debian option here.
-    if "nocheck" in os.environ.get("DEB_BUILD_OPTIONS", "").split():
-        print("Skipped all tests as per DEB_BUILD_OPTIONS environment.")
-        sys.exit(0)
-
-    goHome()
+def parseOptions():
+    # There are freaking many options to honor,
+    # pylint: disable=too-many-branches,too-many-statements
 
     parser = OptionParser()
 
@@ -192,6 +187,17 @@ The standard CPython3.6 test suite. Execute this for all corner cases to be
 covered. With Python 2.x these are not run. Default is %default."""
     )
 
+    parser.add_option(
+        "--skip-cpython37-tests",
+        action  = "store_false",
+        dest    = "cpython37",
+        default = True,
+        help    = """\
+The standard CPython3.6 test suite. Execute this for all corner cases to be
+covered. With Python 2.x these are not run. Default is %default."""
+    )
+
+
     parser.add_option(
         "--skip-other-cpython-tests",
         action  = "store_true",
@@ -242,15 +248,6 @@ Do not use Python2.6 even if available on the system. Default is %default."""
 Do not use Python2.7 even if available on the system. Default is %default."""
     )
 
-    parser.add_option(
-        "--no-python3.2",
-        action  = "store_true",
-        dest    = "no32",
-        default = False,
-        help    = """\
-Do not use Python3.2 even if available on the system. Default is %default."""
-    )
-
     parser.add_option(
         "--no-python3.3",
         action  = "store_true",
@@ -287,6 +284,15 @@ Do not use Python3.5 even if available on the system. Default is %default."""
 Do not use Python3.6 even if available on the system. Default is %default."""
     )
 
+    parser.add_option(
+        "--no-python3.7",
+        action  = "store_true",
+        dest    = "no37",
+        default = False,
+        help    = """\
+Do not use Python3.6 even if available on the system. Default is %default."""
+    )
+
     parser.add_option(
         "--coverage",
         action  = "store_true",
@@ -309,8 +315,6 @@ Make a coverage analysis, that does not really check. Default is %default."""
             options.no26 = True
         if sys.version_info[0:2] != (2,7):
             options.no27 = True
-        if sys.version_info[0:2] != (3,2):
-            options.no32 = True
         if sys.version_info[0:2] != (3,3):
             options.no33 = True
         if sys.version_info[0:2] != (3,4):
@@ -319,6 +323,8 @@ Make a coverage analysis, that does not really check. Default is %default."""
             options.no35 = True
         if sys.version_info[0:2] != (3,6):
             options.no36 = True
+        if sys.version_info[0:2] != (3,7):
+            options.no37 = True
 
     if options.cpython_no_other:
         if sys.version_info[0:2] != (2,6):
@@ -335,6 +341,8 @@ Make a coverage analysis, that does not really check. Default is %default."""
             options.cpython35 = False
         if sys.version_info[0:2] != (3,6):
             options.cpython36 = False
+        if sys.version_info[0:2] != (3,7):
+            options.cpython37 = False
 
     if options.cpython_none:
         options.cpython26 = False
@@ -344,10 +352,95 @@ Make a coverage analysis, that does not really check. Default is %default."""
         options.cpython34 = False
         options.cpython35 = False
         options.cpython36 = False
+        options.cpython37 = False
 
     if options.coverage and os.path.exists(".coverage"):
         os.unlink(".coverage")
 
+    return options
+
+
+def publishCoverageData():
+    def copyToGlobalCoverageData(source, target):
+        coverage_dir = os.environ.get("COVERAGE_DIR", None)
+
+        if coverage_dir is None:
+            return
+
+        subprocess.check_call(
+            (
+                "scp",
+                source,
+                os.path.join(
+                    coverage_dir,
+                    target
+                )
+            )
+        )
+
+    if os.name == "nt":
+        suffix = "win"
+    else:
+        import platform
+        suffix = platform.uname()[0] + '.' + platform.uname()[4]
+
+    with open("data.coverage", 'w') as data_file:
+        source_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
+
+        with withDirectoryChange(source_dir):
+            nuitka_id = check_output("git rev-parse HEAD".split())
+        nuitka_id = nuitka_id.strip()
+
+        if sys.version_info > (3,):
+            nuitka_id = nuitka_id.decode()
+
+        data_file.write("NUITKA_SOURCE_DIR=%r\n" % source_dir)
+        data_file.write("NUITKA_COMMIT=%r\n" % nuitka_id)
+
+
+    copyToGlobalCoverageData("data.coverage", "meta.coverage." + suffix)
+
+    def makeCoverageRelative(filename):
+        """ Normalize coverage data.
+
+        """
+
+        with open(filename) as input_file:
+            data = input_file.read()
+
+        data = data.replace(
+            (os.path.abspath('.') + os.path.sep).replace('\\', "\\\\"),
+            ""
+        )
+
+        if os.path.sep != '/':
+            data.replace(os.path.sep, '/')
+
+        with open(filename, 'w') as output_file:
+            output_file.write(data)
+
+    coverage_file = os.environ.get("COVERAGE_FILE", ".coverage")
+
+    makeCoverageRelative(coverage_file)
+    copyToGlobalCoverageData(coverage_file, "data.coverage." + suffix)
+
+
+def main():
+    # There are many cases to deal with,
+    # pylint: disable=too-many-branches,too-many-statements
+
+    # Lets honor this Debian option here.
+    if "nocheck" in os.environ.get("DEB_BUILD_OPTIONS", "").split():
+        print("Skipped all tests as per DEB_BUILD_OPTIONS environment.")
+        sys.exit(0)
+
+    # Make sure our resolving of "python2" to "python" doesn't get in the way.
+    os.environ["PYTHON_DISALLOW_AMBIGUOUS_VERSION"] = '0'
+
+    goHome()
+
+    options = parseOptions()
+
     # Add the local bin directory to search path start.
     os.environ["PATH"] = \
       os.path.join(
@@ -366,8 +459,6 @@ Make a coverage analysis, that does not really check. Default is %default."""
             return False
         if command == "python2.7" and options.no27:
             return False
-        if command == "python3.2" and options.no32:
-            return False
         if command == "python3.3" and options.no33:
             return False
         if command == "python3.4" and options.no34:
@@ -376,6 +467,8 @@ Make a coverage analysis, that does not really check. Default is %default."""
             return False
         if command == "python3.6" and options.no36:
             return False
+        if command == "python3.7" and options.no37:
+            return False
 
         # Shortcuts for python versions, also needed for Windows as it won't have
         # the version number in the Python binaries at all.
@@ -383,8 +476,6 @@ Make a coverage analysis, that does not really check. Default is %default."""
             return True
         if command == "python2.7" and sys.version_info[0:2] == (2,7):
             return True
-        if command == "python3.2" and sys.version_info[0:2] == (3,2):
-            return True
         if command == "python3.3" and sys.version_info[0:2] == (3,3):
             return True
         if command == "python3.4" and sys.version_info[0:2] == (3,4):
@@ -393,6 +484,8 @@ Make a coverage analysis, that does not really check. Default is %default."""
             return True
         if command == "python3.6" and sys.version_info[0:2] == (3,6):
             return True
+        if command == "python3.7" and sys.version_info[0:2] == (3,7):
+            return True
 
         path = os.environ[ "PATH" ]
 
@@ -514,7 +607,7 @@ Make a coverage analysis, that does not really check. Default is %default."""
         # At least one Debian Jessie, these versions won't have lxml installed, so
         # don't run them there. Also these won't be very version dependent in their
         # results.
-        if use_python != "python2.6" and use_python != "python3.2":
+        if use_python != "python2.6":
             if options.optimization_tests:
                 print("Running the optimizations tests with options '%s' with %s:" % (flags, use_python))
                 setExtraFlags(where, "optimizations", flags)
@@ -554,8 +647,8 @@ Make a coverage analysis, that does not really check. Default is %default."""
         if "--debug" not in flags:
             # Not running the Python 3.2 test suite with CPython2.6, as that's about
             # the same as CPython2.7 and won't have any new insights.
-            if use_python != "python2.6" and \
-               use_python != "python2.7" or not options.coverage:
+            if use_python not in ("python2.6", "python2.7") or \
+               not options.coverage:
                 if os.path.exists("./tests/CPython32/run_all.py"):
                     if options.cpython32:
                         setExtraFlags(where, "32tests", flags)
@@ -581,7 +674,7 @@ Make a coverage analysis, that does not really check. Default is %default."""
                 else:
                     print("The CPython3.4 tests are not present, not run.")
 
-            # Running the Python 3.4 test suite only with CPython3.x.
+            # Running the Python 3.5 test suite only with CPython3.x.
             if not use_python.startswith("python2"):
                 if os.path.exists("./tests/CPython35/run_all.py"):
                     if options.cpython35:
@@ -590,7 +683,7 @@ Make a coverage analysis, that does not really check. Default is %default."""
                 else:
                     print("The CPython3.5 tests are not present, not run.")
 
-            # Running the Python 3.4 test suite only with CPython3.x.
+            # Running the Python 3.6 test suite only with CPython3.x.
             if not use_python.startswith("python2"):
                 if os.path.exists("./tests/CPython36/run_all.py"):
                     if options.cpython36:
@@ -599,23 +692,25 @@ Make a coverage analysis, that does not really check. Default is %default."""
                 else:
                     print("The CPython3.6 tests are not present, not run.")
 
+            # Running the Python 3.7 test suite only with CPython3.x.
+            if not use_python.startswith("python2"):
+                if os.path.exists("./tests/CPython36/run_all.py"):
+                    if options.cpython36:
+                        setExtraFlags(where, "37tests", flags)
+                        executeSubTest("./tests/CPython37/run_all.py search")
+                else:
+                    print("The CPython3.7 tests are not present, not run.")
+
         if "NUITKA_EXTRA_OPTIONS" in os.environ:
             del os.environ[ "NUITKA_EXTRA_OPTIONS" ]
 
     assert checkExecutableCommand("python2.6") or \
            checkExecutableCommand("python2.7") or \
-           checkExecutableCommand("python3.2") or \
            checkExecutableCommand("python3.3") or \
            checkExecutableCommand("python3.4") or \
            checkExecutableCommand("python3.5") or \
-           checkExecutableCommand("python3.6")
-
-    # Just the quick syntax test, full tests are run later.
-    if checkExecutableCommand("python3.2"):
-        executeSubTest(
-            "./bin/nuitka --python-version=3.2 --version",
-            hide_output = True
-        )
+           checkExecutableCommand("python3.6") or \
+           checkExecutableCommand("python3.7")
 
     if checkExecutableCommand("python2.6"):
         execute_tests("python2.6-debug", "python2.6", "--debug")
@@ -627,11 +722,6 @@ Make a coverage analysis, that does not really check. Default is %default."""
     else:
         print("Cannot execute tests with Python 2.7, disabled or not installed.")
 
-    if checkExecutableCommand("python3.2"):
-        execute_tests("python3.2-debug", "python3.2", "--debug")
-    else:
-        print("Cannot execute tests with Python 3.2, disabled or not installed.")
-
     if checkExecutableCommand("python2.6"):
         execute_tests("python2.6-nodebug", "python2.6", "")
     else:
@@ -642,11 +732,6 @@ Make a coverage analysis, that does not really check. Default is %default."""
     else:
         print("Cannot execute tests with Python 2.7, disabled or not installed.")
 
-    if checkExecutableCommand("python3.2"):
-        execute_tests("python3.2-nodebug", "python3.2", "")
-    else:
-        print("Cannot execute tests with Python 3.2, disabled or not installed.")
-
     if checkExecutableCommand("python3.3"):
         execute_tests("python3.3-nodebug", "python3.3", "")
     else:
@@ -667,67 +752,13 @@ Make a coverage analysis, that does not really check. Default is %default."""
     else:
         print("Cannot execute tests with Python 3.6, disabled or not installed.")
 
+    if checkExecutableCommand("python3.7"):
+        execute_tests("python3.7-nodebug", "python3.7", "")
+    else:
+        print("Cannot execute tests with Python 3.7, disabled or not installed.")
+
     if options.coverage:
-        def copyToGlobalCoverageData(source, target):
-            coverage_dir = os.environ.get("COVERAGE_DIR", None)
-
-            if coverage_dir is None:
-                return
-
-            assert subprocess.call(
-                (
-                    "C:\\MinGW\\msys\\1.0\\bin\\scp.exe"
-                       if os.name == "nt" else
-                    "scp",
-                    source,
-                    os.path.join(
-                        coverage_dir,
-                        target
-                    )
-                )
-            ) == 0
-
-        if os.name == "nt":
-            suffix = "win"
-        else:
-            import platform
-            suffix = platform.uname()[0] + '.' + platform.uname()[4]
-
-        with open("data.coverage", 'w') as data_file:
-            source_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
-
-            nuitka_id = check_output(
-                "cd '%s'; git rev-parse HEAD" % source_dir,
-                shell = True
-            )
-            nuitka_id = nuitka_id.strip()
-
-            if sys.version_info > (3,):
-                nuitka_id = nuitka_id.decode()
-
-            data_file.write("NUITKA_COMMIT='%s'\n" % nuitka_id)
-
-        copyToGlobalCoverageData("data.coverage", "data.coverage." + suffix)
-
-        def makeCoverageRelative(filename):
-            """ Normalize coverage data.
-
-            """
-
-            with open(filename) as input_file:
-                data = input_file.read()
-
-            data = data.replace(os.path.abspath('.') + os.path.sep, "")
-            if os.path.sep != '/':
-                data.replace(os.path.sep, '/')
-
-            with open(filename, 'w') as output_file:
-                output_file.write(data)
-
-        coverage_file = os.environ.get("COVERAGE_FILE", ".coverage")
-
-        makeCoverageRelative(coverage_file)
-        copyToGlobalCoverageData(coverage_file, ".coverage." + suffix)
+        publishCoverageData()
 
     print("OK.")
 
