@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -37,17 +37,13 @@ from nuitka.nodes.BuiltinIteratorNodes import (
 )
 from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
 from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinRef
-from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.CoroutineNodes import (
     ExpressionCoroutineObjectBody,
     ExpressionMakeCoroutineObject
 )
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionBody,
-    ExpressionFunctionCall,
     ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
@@ -56,27 +52,31 @@ from nuitka.nodes.GeneratorNodes import (
     ExpressionMakeGeneratorObject,
     StatementGeneratorReturnNone
 )
-from nuitka.nodes.ParameterSpecs import ParameterSpec
+from nuitka.nodes.LocalsDictNodes import StatementSetLocalsDictionary
+from nuitka.nodes.OutlineNodes import ExpressionOutlineFunction
 from nuitka.nodes.ReturnNodes import StatementReturn, StatementReturnNone
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableNameRef
+    ExpressionVariableNameRef,
+    ExpressionVariableRef
 )
 from nuitka.PythonVersions import python_version
+from nuitka.specs.ParameterSpecs import ParameterSpec
 
-from .Helpers import (
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import (
     buildFrameNode,
     buildNode,
     buildNodeList,
     detectFunctionBodyKind,
     extractDocFromBody,
     getKind,
-    makeDictCreationOrConstant,
+    makeCallNode,
+    makeDictCreationOrConstant2,
     makeStatementsSequenceFromStatement,
     mangleName
 )
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
-from .SyntaxErrors import raiseSyntaxError
 
 
 def _insertFinalReturnStatement(function_statements_body, return_statement):
@@ -95,8 +95,45 @@ def _insertFinalReturnStatement(function_statements_body, return_statement):
     return function_statements_body
 
 
+def _insertInitialSetLocalsDictStatement(function_body, function_statements_body):
+    locals_statement = StatementSetLocalsDictionary(
+        locals_scope = function_body.getFunctionLocalsScope(),
+        source_ref   = function_body.source_ref
+    )
+
+    if function_statements_body is None:
+        function_statements_body = makeStatementsSequenceFromStatement(
+            statement = locals_statement
+        )
+    else:
+        function_statements_body.setStatements(
+            (
+                locals_statement,
+            ) + function_statements_body.getStatements()
+        )
+
+    return function_statements_body
+
+
+def _injectDecorator(decorators, inject, acceptable, source_ref):
+    assert type(inject) is str
+    assert type(acceptable) is tuple
+
+    for decorator in decorators:
+        if decorator.isExpressionVariableNameRef() and \
+           decorator.getVariableName() in acceptable:
+            break
+    else:
+        decorators.append(
+            makeExpressionBuiltinRef(
+                builtin_name = inject,
+                source_ref   = source_ref
+            )
+        )
+
+
 def buildFunctionNode(provider, node, source_ref):
-    # Functions have way too many details, pylint: disable=too-many-branches,too-many-locals
+    # Functions have way too many details, pylint: disable=too-many-locals
 
     assert getKind(node) == "FunctionDef"
 
@@ -106,7 +143,7 @@ def buildFunctionNode(provider, node, source_ref):
         nodes = function_statement_nodes
     )
 
-    outer_body, function_body, code_object = buildFunctionWithParsing(
+    function_body, code_body, code_object = buildFunctionWithParsing(
         provider      = provider,
         function_kind = function_kind,
         name          = node.name,
@@ -116,33 +153,41 @@ def buildFunctionNode(provider, node, source_ref):
         source_ref    = source_ref
     )
 
-    if function_kind == "Function":
-        code_body = function_body
-    elif function_kind == "Generator":
-        code_body = ExpressionGeneratorObjectBody(
-            provider   = function_body,
-            name       = node.name,
-            flags      = flags,
-            source_ref = source_ref
-        )
+    if function_kind in ("Generator", "Coroutine"):
+        if function_kind == "Coroutine":
+            code_body = ExpressionCoroutineObjectBody(
+                provider   = function_body,
+                name       = node.name,
+                flags      = flags,
+                source_ref = source_ref
+            )
+
+            maker_class = ExpressionMakeCoroutineObject
+        else:
+            code_body = ExpressionGeneratorObjectBody(
+                provider   = function_body,
+                name       = node.name,
+                flags      = flags,
+                source_ref = source_ref
+            )
+
+            maker_class = ExpressionMakeGeneratorObject
+
         code_body.qualname_provider = provider
 
         for variable in function_body.getVariables():
             code_body.getVariableForReference(variable.getName())
-    else:
-        assert False, function_kind
 
-    if function_kind == "Generator":
         function_body.setBody(
             makeStatementsSequenceFromStatement(
                 statement = StatementReturn(
-                    expression = ExpressionMakeGeneratorObject(
-                        generator_ref = ExpressionFunctionRef(
+                    expression = maker_class(
+                        ExpressionFunctionRef(
                             function_body = code_body,
                             source_ref    = source_ref
                         ),
-                        code_object   = code_object,
-                        source_ref    = source_ref
+                        code_object = code_object,
+                        source_ref  = source_ref
                     ),
                     source_ref = source_ref
                 )
@@ -184,6 +229,12 @@ def buildFunctionNode(provider, node, source_ref):
             )
         )
 
+    if "has_exec" in flags:
+        function_statements_body = _insertInitialSetLocalsDictStatement(
+            function_body            = code_body,
+            function_statements_body = function_statements_body,
+        )
+
     if function_statements_body.isStatementsFrame():
         function_statements_body = makeStatementsSequenceFromStatement(
             statement = function_statements_body
@@ -197,7 +248,7 @@ def buildFunctionNode(provider, node, source_ref):
 
     function_creation = ExpressionFunctionCreation(
         function_ref = ExpressionFunctionRef(
-            function_body = outer_body,
+            function_body = function_body,
             source_ref    = source_ref
         ),
         code_object  = code_object,
@@ -211,49 +262,32 @@ def buildFunctionNode(provider, node, source_ref):
 
     # CPython made these optional, but secretly applies them when it does
     # "class __new__".  We add them earlier, so our optimization will see it.
-    if node.name == "__new__" and \
+    if python_version < 300 and \
+       node.name == "__new__" and \
        provider.isExpressionClassBody():
+        _injectDecorator(decorators, "staticmethod", ("staticmethod", "classmethod"), source_ref)
 
-        for decorator in decorators:
-            if decorator.isExpressionVariableNameRef() and \
-               decorator.getVariableName() == "staticmethod":
-                break
-        else:
-            decorators.append(
-                makeExpressionBuiltinRef(
-                    builtin_name = "staticmethod",
-                    source_ref   = source_ref
-                )
-            )
-
+    # Add the "classmethod" decorator to __init_subclass__ methods if not provided.
     if python_version >= 360 and \
        node.name == "__init_subclass__" and \
        provider.isExpressionClassBody():
+        _injectDecorator(decorators, "classmethod", ("classmethod",), source_ref)
 
-        for decorator in decorators:
-            if decorator.isExpressionVariableNameRef() and \
-               decorator.getVariableName() == "classmethod":
-                break
-        else:
-            decorators.append(
-                makeExpressionBuiltinRef(
-                    builtin_name = "classmethod",
-                    source_ref   = source_ref
-                )
-            )
+    if python_version >= 370 and \
+       node.name == "__class_getitem__" and \
+       provider.isExpressionClassBody():
+        _injectDecorator(decorators, "classmethod", ("classmethod",), source_ref)
 
     decorated_function = function_creation
     for decorator in decorators:
-        decorated_function = ExpressionCallNoKeywords(
-            called     = decorator,
-            args       = ExpressionMakeTuple(
-                elements   = (decorated_function,),
-                source_ref = source_ref
-            ),
-            source_ref = decorator.getSourceReference()
+        decorated_function = makeCallNode(
+            decorator,
+            decorated_function,
+            decorator.getSourceReference()
         )
 
     result = StatementAssignmentVariableName(
+        provider      = provider,
         variable_name = mangleName(node.name, provider),
         source        = decorated_function,
         source_ref    = source_ref
@@ -281,8 +315,8 @@ def buildAsyncFunctionNode(provider, node, source_ref):
         provider      = provider,
         function_kind = function_kind,
         name          = node.name,
-        flags         = (),
         function_doc  = function_doc,
+        flags         = (),
         node          = node,
         source_ref    = source_ref
     )
@@ -302,6 +336,8 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             source_ref = source_ref
         )
 
+    function_body.qualname_provider = provider
+
     for variable in creator_function_body.getVariables():
         function_body.getVariableForReference(variable.getName())
 
@@ -392,17 +428,14 @@ def buildAsyncFunctionNode(provider, node, source_ref):
 
     decorated_function = function_creation
     for decorator in decorators:
-        decorated_function = ExpressionCallNoKeywords(
-            called     = decorator,
-            args       = ExpressionMakeTuple(
-                elements   = (decorated_function,),
-                source_ref = source_ref
-            ),
-            source_ref = decorator.getSourceReference()
+        decorated_function = makeCallNode(
+            decorator,
+            decorated_function,
+            decorator.getSourceReference()
         )
 
-
     result = StatementAssignmentVariableName(
+        provider      = provider,
         variable_name = mangleName(node.name, provider),
         source        = decorated_function,
         source_ref    = source_ref
@@ -431,17 +464,12 @@ def buildParameterKwDefaults(provider, node, function_body, source_ref):
             for kw_only_name, kw_default in \
               zip(kw_only_names, node.args.kw_defaults):
                 if kw_default is not None:
-                    keys.append(
-                        makeConstantRefNode(
-                            constant   = kw_only_name,
-                            source_ref = source_ref
-                        )
-                    )
+                    keys.append(kw_only_name)
                     values.append(
                         buildNode(provider, kw_default, source_ref)
                     )
 
-            kw_defaults = makeDictCreationOrConstant(
+            kw_defaults = makeDictCreationOrConstant2(
                 keys       = keys,
                 values     = values,
                 source_ref = source_ref
@@ -472,13 +500,7 @@ def buildParameterAnnotations(provider, node, source_ref):
     values = []
 
     def addAnnotation(key, value):
-        keys.append(
-            makeConstantRefNode(
-                constant      = mangle(key),
-                source_ref    = source_ref,
-                user_provided = True
-            )
-        )
+        keys.append(mangle(key))
         values.append(value)
 
     def extractArg(arg):
@@ -534,7 +556,7 @@ def buildParameterAnnotations(provider, node, source_ref):
         )
 
     if keys:
-        return makeDictCreationOrConstant(
+        return makeDictCreationOrConstant2(
             keys       = keys,
             values     = values,
             source_ref = source_ref
@@ -543,13 +565,159 @@ def buildParameterAnnotations(provider, node, source_ref):
         return None
 
 
+def _wrapFunctionWithSpecialNestedArgs(name, outer_body, parameters, special_args, source_ref):
+    inner_name = name.strip("<>") + "$inner"
+    iter_vars = []
+
+    values = []
+
+    statements = []
+
+    def unpackFrom(source, arg_names):
+        accesses = []
+
+        sub_special_index = 0
+
+        iter_var = outer_body.allocateTempVariable(None, "arg_iter_%d" % len(iter_vars))
+        iter_vars.append(iter_var)
+
+        statements.append(
+            StatementAssignmentVariable(
+                variable   = iter_var,
+                source     = ExpressionBuiltinIter1(
+                    value      = source,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            )
+        )
+
+        for element_index, arg_name in enumerate(arg_names):
+            if getKind(arg_name) == "Name":
+                arg_var = outer_body.createProvidedVariable(arg_name.id)
+                outer_body.registerProvidedVariable(arg_var)
+
+                statements.append(
+                    StatementAssignmentVariable(
+                        variable   = arg_var,
+                        source     = ExpressionSpecialUnpack(
+                            value      = ExpressionTempVariableRef(
+                                variable   = iter_var,
+                                source_ref = source_ref
+                            ),
+                            count      = element_index + 1,
+                            expected   = len(arg_names),
+                            starred    = False,
+                            source_ref = source_ref
+                        ),
+                        source_ref = source_ref
+                    )
+                )
+
+                accesses.append(
+                    ExpressionVariableRef(
+                        variable   = arg_var,
+                        source_ref = source_ref
+                    )
+                )
+            elif getKind(arg_name) == "Tuple":
+                accesses.extend(
+                    unpackFrom(
+                        source    = ExpressionSpecialUnpack(
+                            value      = ExpressionTempVariableRef(
+                                variable   = iter_var,
+                                source_ref = source_ref
+                            ),
+                            count      = element_index + 1,
+                            expected   = len(arg_names),
+                            starred    = False,
+                            source_ref = source_ref
+                        ),
+                        arg_names = arg_name.elts
+                    )
+                )
+
+                sub_special_index += 1
+            else:
+                assert False, arg_name
+
+        statements.append(
+            StatementSpecialUnpackCheck(
+                iterator   = ExpressionTempVariableRef(
+                    variable   = iter_var,
+                    source_ref = source_ref
+                ),
+                count      = len(arg_names),
+                source_ref = source_ref
+            )
+        )
+
+        return accesses
+
+    for arg_name in parameters.getParameterNames():
+        if arg_name.startswith('.'):
+            source = ExpressionVariableNameRef(
+                provider      = outer_body,
+                variable_name = arg_name,
+                source_ref    = source_ref
+            )
+
+            values.extend(
+                unpackFrom(source, special_args[arg_name])
+            )
+        else:
+            values.append(
+                ExpressionVariableNameRef(
+                    provider      = outer_body,
+                    variable_name = arg_name,
+                    source_ref    = source_ref
+                )
+            )
+
+    code_body = ExpressionOutlineFunction(
+        provider   = outer_body,
+        name       = inner_name,
+        source_ref = source_ref
+    )
+
+    statements.append(
+        StatementReturn(
+            expression = code_body,
+            source_ref = source_ref
+        )
+    )
+
+    outer_body.setBody(
+        makeStatementsSequenceFromStatement(
+            statement = makeTryFinallyStatement(
+                provider   = outer_body,
+                tried      = statements,
+                final      = [
+                    StatementReleaseVariable(
+                        variable   = variable,
+                        source_ref = source_ref
+                    )
+                    for variable in
+                    sorted(
+                        outer_body.getTempVariables(),
+                        key = lambda variable: variable.getName()
+                    )
+                ],
+                source_ref = source_ref,
+                public_exc = False
+            )
+        )
+    )
+
+    return code_body
+
+
 def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                              node, source_ref):
     # This contains a complex re-formulation for nested parameter functions.
     # pylint: disable=too-many-locals
 
     kind = getKind(node)
-
     assert kind in ("FunctionDef", "Lambda", "AsyncFunctionDef"), "unsupported for kind " + kind
 
     def extractArg(arg):
@@ -632,187 +800,24 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
         source_ref = source_ref
     )
 
+    # Wrap if necessary for special nested arguments.
     if special_args:
-        inner_name = name.strip("<>") + "$inner"
-        inner_arg_names = []
-        iter_vars = []
-
-        values = []
-
-        statements = []
-
-        def unpackFrom(source, arg_names):
-            accesses = []
-
-            sub_special_index = 0
-
-            iter_var = outer_body.allocateTempVariable(None, "arg_iter_%d" % len(iter_vars))
-            iter_vars.append(iter_var)
-
-            statements.append(
-                StatementAssignmentVariable(
-                    variable   = iter_var,
-                    source     = ExpressionBuiltinIter1(
-                        value      = source,
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref
-                )
-            )
-
-            for element_index, arg_name in enumerate(arg_names):
-                if getKind(arg_name) == "Name":
-                    inner_arg_names.append(arg_name.id)
-
-                    arg_var = outer_body.allocateTempVariable(None, "tmp_" + arg_name.id)
-
-                    statements.append(
-                        StatementAssignmentVariable(
-                            variable   = arg_var,
-                            source     = ExpressionSpecialUnpack(
-                                value      = ExpressionTempVariableRef(
-                                    variable   = iter_var,
-                                    source_ref = source_ref
-                                ),
-                                count      = element_index + 1,
-                                expected   = len(arg_names),
-                                source_ref = source_ref
-                            ),
-                            source_ref = source_ref
-                        )
-                    )
-
-                    accesses.append(
-                        ExpressionTempVariableRef(
-                            variable   = arg_var,
-                            source_ref = source_ref
-                        )
-                    )
-                elif getKind(arg_name) == "Tuple":
-                    accesses.extend(
-                        unpackFrom(
-                            source    = ExpressionSpecialUnpack(
-                                value      = ExpressionTempVariableRef(
-                                    variable   = iter_var,
-                                    source_ref = source_ref
-                                ),
-                                count      = element_index + 1,
-                                expected   = len(arg_names),
-                                source_ref = source_ref
-                            ),
-                            arg_names = arg_name.elts
-                        )
-                    )
-
-                    sub_special_index += 1
-                else:
-                    assert False, arg_name
-
-            statements.append(
-                StatementSpecialUnpackCheck(
-                    iterator   = ExpressionTempVariableRef(
-                        variable   = iter_var,
-                        source_ref = source_ref
-                    ),
-                    count      = len(arg_names),
-                    source_ref = source_ref
-                )
-            )
-
-            return accesses
-
-        for arg_name in parameters.getParameterNames():
-            if arg_name.startswith('.'):
-                source = ExpressionVariableNameRef(
-                    variable_name = arg_name,
-                    source_ref    = source_ref
-                )
-
-                values.extend(
-                    unpackFrom(source, special_args[arg_name])
-                )
-            else:
-                values.append(
-                    ExpressionVariableNameRef(
-                        variable_name = arg_name,
-                        source_ref    = source_ref
-                    )
-                )
-
-                inner_arg_names.append(arg_name)
-
-        inner_parameters = ParameterSpec(
-            ps_name          = inner_name,
-            ps_normal_args   = inner_arg_names,
-            ps_kw_only_args  = (),
-            ps_list_star_arg = None,
-            ps_dict_star_arg = None,
-            ps_default_count = None
+        code_body = _wrapFunctionWithSpecialNestedArgs(
+            name         = name,
+            outer_body   = outer_body,
+            parameters   = parameters,
+            special_args = special_args,
+            source_ref   = source_ref
         )
 
-        function_body = ExpressionFunctionBody(
-            provider   = outer_body,
-            name       = inner_name,
-            flags      = flags,
-            doc        = function_doc,
-            parameters = inner_parameters,
-            source_ref = source_ref
-        )
-
-        statements.append(
-            StatementReturn(
-                ExpressionFunctionCall(
-                    function   = ExpressionFunctionCreation(
-                        function_ref = ExpressionFunctionRef(
-                            function_body = function_body,
-                            source_ref    = source_ref
-                        ),
-                        code_object  = code_object,
-                        defaults     = (),
-                        kw_defaults  = None,
-                        annotations  = None,
-                        source_ref   = source_ref
-                    ),
-                    values     = values,
-                    source_ref = source_ref
-                ),
-                source_ref = source_ref
-            )
-        )
-
-        outer_body.setBody(
-            makeStatementsSequenceFromStatement(
-                statement = makeTryFinallyStatement(
-                    provider   = outer_body,
-                    tried      = statements,
-                    final      = [
-                        StatementReleaseVariable(
-                            variable   = variable,
-                            source_ref = source_ref
-                        )
-                        for variable in
-                        sorted(
-                            outer_body.getTempVariables(),
-                            key = lambda variable: variable.getName()
-                        )
-                    ],
-                    source_ref = source_ref,
-                    public_exc = False
-                )
-            )
-        )
     else:
-        function_body = outer_body
+        code_body = outer_body
 
-    return outer_body, function_body, code_object
+    return outer_body, code_body, code_object
 
 
 def addFunctionVariableReleases(function):
-    assert function.isExpressionFunctionBody() or \
-           function.isExpressionClassBody() or \
-           function.isExpressionGeneratorObjectBody() or \
-           function.isExpressionCoroutineObjectBody() or \
-           function.isExpressionAsyncgenObjectBody()
+    assert function.isExpressionFunctionBodyBase()
 
     releases = []
 
