@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -50,6 +50,7 @@ tmp_dir = getTempDir()
 PACKAGE_LIST = (
     "nuitka",
     "nuitka/nodes",
+    "nuitka/specs",
     "nuitka/nodes/shapes",
     "nuitka/tree",
     "nuitka/importing",
@@ -188,7 +189,7 @@ def executePASS1():
 
     my_print("Compiling '%s'." % nuitka_main_path)
 
-    shutil.copyfile(nuitka_main_path, "nuitka.py")
+    shutil.copyfile(nuitka_main_path, "nuitka-runner.py")
 
     command = [
         os.environ["PYTHON"],
@@ -197,7 +198,7 @@ def executePASS1():
         "--plugin-enable=pylint-warnings",
         "--output-dir=.",
         "--python-flag=-S",
-        "nuitka.py"
+        "nuitka-runner.py"
     ]
     command += os.environ.get("NUITKA_EXTRA_OPTIONS", "").split()
 
@@ -208,6 +209,8 @@ def executePASS1():
     if result != 0:
         sys.exit(result)
 
+    shutil.move("nuitka-runner.exe", "nuitka.exe")
+
     scons_inline_copy_path = os.path.join(
         base_dir,
         "nuitka",
